# Author: Carsten Sachse 17-Feb-2011
# Copyright: EMBL (2010 - 2018)
# License: see license.txt for details
"""
Spring (Single Particle Reconstruction from Images of kNown Geometries) - 
suite of programs for processing cryo-EM images of helices
"""
from argparse import ArgumentParser
from PyQt4.QtCore import SIGNAL
from PyQt4.QtGui import QApplication, QMainWindow, QWidget, QStackedWidget, QGridLayout, QComboBox, QLabel, QFileDialog
from spring.csinfrastr.csdatabase import SpringDataBase, RefinementCycleTable, RefinementCycleHelixTable, \
    RefinementCycleSegmentTable, RefinementCycleSegmentSubunitTable, refine_base, SegmentTable, HelixTable, \
    CtfMicrographTable, CtfFindMicrographTable, CtfTiltMicrographTable, SubunitTable, base, GridTable, GridRefineTable, \
    grid_base
from spring.csinfrastr.csfeatures import Features
from spring.csinfrastr.cslogger import GetMetaData
from spring.springgui.spring2d_launch import Segment2dGui
from spring.springgui.spring3d_launch import Spring3dGui
from spring.springgui.spring_menu import GuiActions
from spring.springgui.springmicrograph import SpringMicrographGui
import os
import spring.springgui.spring2d_launch
import spring.springgui.spring3d_launch
import spring.springgui.springmicrograph
import sys


class SpringGuiTop(QWidget):
    def __init__(self, parent=None):
        QWidget.__init__(self)

        self.suitename = os.path.basename(sys.argv[0])
        self.setWindowTitle('%s' %self.suitename)

        self.subPackageComboBox = QComboBox()
        self.subPackageComboBox.addItems(['Springmicrograph', 'Spring2d', 'Spring3d'])
        
        
class SpringGui(QWidget):
    def __init__(self, parent=None):
        QWidget.__init__(self, parent)
        
        self.springguitop = SpringGuiTop()
        
        self.stackedWidget = QStackedWidget()
        self.springMicrographW = SpringMicrographGui()
        self.spring2dW = Segment2dGui()
        self.spring3dW = Spring3dGui()
        
        self.suite_description = QLabel()
        self.springmicrograph_description = '{0}'.format(spring.springgui.springmicrograph.__doc__)
        self.spring2d_description = '{0}'.format(spring.springgui.spring2d_launch.__doc__)
        self.spring3d_description = '{0}'.format(spring.springgui.spring3d_launch.__doc__)
        
        self.suite_description.setText(self.springmicrograph_description)

        self.stackedWidget.addWidget(self.springMicrographW)
        self.stackedWidget.addWidget(self.spring2dW)
        self.stackedWidget.addWidget(self.spring3dW)
        
        layout = QGridLayout()
        layout.addWidget(self.springguitop.subPackageComboBox, 0, 0, 1, 1)
        layout.addWidget(self.suite_description, 0, 1, 1, 1)

        layout.addWidget(self.stackedWidget, 1, 0, 4, 4)
        layout.setColumnStretch(0, 0)
        self.setLayout(layout)
        
        self.connect(self.springguitop.subPackageComboBox, SIGNAL('currentIndexChanged(QString)'), self.setWidgetStack)
    
    def setWidgetStack(self, text):
        if text == 'Springmicrograph':
            self.stackedWidget.setCurrentIndex(0)
            self.suite_description.setText(self.springmicrograph_description)
        elif text == 'Spring2d':
            self.stackedWidget.setCurrentIndex(1)
            self.suite_description.setText(self.spring2d_description)
        elif text == 'Spring3d':
            self.stackedWidget.setCurrentIndex(2)
            self.suite_description.setText(self.spring3d_description)
        

class SpringMain(QMainWindow, GuiActions):
    def __init__(self, parent=None):
        QMainWindow.__init__(self, parent)

        self.package = 'emspring'

        self.guiWindow = SpringGui()
        
        # inherit body of layout of actions
        self.setupUi(self)
        self.connect(self.beginnerAction, SIGNAL('triggered()'), self.changeToBeginnerDisplay)
        self.connect(self.intermediateAction, SIGNAL('triggered()'), self.changeToIntermediateDisplay)
        self.connect(self.expertAction, SIGNAL('triggered()'), self.changeToExpertDisplay)
        self.connect(self.openAction, SIGNAL('triggered()'), self.openParFile)
        self.connect(self.saveAction, SIGNAL('triggered()'), self.saveParFile)
        
        self.setCentralWidget(self.guiWindow)
        self.changeToBeginnerDisplay()


    def getCurrentTab(self):
        currentTab = self.guiWindow.stackedWidget.currentWidget().tabWidget.currentWidget().springTab

        return currentTab


    def openParFile(self):
        fname = QFileDialog.getOpenFileName(self , 'Choose file', '.', 'Files ({0})'.format('*.par'))
        if fname != '':
            from spring.csinfrastr.csreadinput import ReadInputFile
            currentTab = self.getCurrentTab()
            currentTab.parset.filename = fname
            inp = ReadInputFile(currentTab.parset)
            currentTab.enterParametersInGui(inp.parameters)
            
        
    def saveParFile(self):
        fname = str(QFileDialog.getSaveFileName(self , 'Save file', '.', '{0}'.format('Parameter text file (*.par);; ' +\
        'Python submission script (*.py)' ))) 
        
        if fname != '':            
            currentTab = self.getCurrentTab()
            currentTab.readParams()
            
        if fname.endswith('par'):
            Features().write_parameters_in_file(currentTab.parset.parameters, fname)
        elif fname.endswith('py'):
            sub_string  = Features().get_submission_script_str(currentTab.parset.progname, currentTab.parset.parameters)
            script = open(fname, 'w')
            script.write(sub_string)
            script.close()
            
            
    def removeTabsFromIndices(self, tab_indices):
        for each_index in tab_indices:
            self.guiWindow.stackedWidget.currentWidget().tabWidget.removeTab(each_index)


    def removeAllTabsFromAllWidgets(self):
        self.currentIndex = self.guiWindow.stackedWidget.currentIndex()
        self.currentTabIndex = self.guiWindow.stackedWidget.currentWidget().tabWidget.currentIndex()
        for each_index in [0, 1, 2]:
            self.guiWindow.stackedWidget.setCurrentIndex(each_index)
            
            tab_indices = list(range(self.guiWindow.stackedWidget.currentWidget().tabWidget.count()))
            tab_indices.reverse()
            self.removeTabsFromIndices(tab_indices)


    def removeTabsInThreeWidgetsFromIndices(self, indices_one, indices_two, indices_three):
        self.guiWindow.stackedWidget.setCurrentIndex(0)
        self.removeTabsFromIndices(indices_one)
        self.guiWindow.stackedWidget.setCurrentIndex(1)
        self.removeTabsFromIndices(indices_two)
        self.guiWindow.stackedWidget.setCurrentIndex(2)
        self.removeTabsFromIndices(indices_three)


    def displayEntriesBasedOnLevel(self, level):
        for eachWidgetIndex in list(range(self.guiWindow.stackedWidget.count())):
            self.guiWindow.stackedWidget.setCurrentIndex(eachWidgetIndex)
            for eachTabIndex in list(range(self.guiWindow.stackedWidget.currentWidget().tabWidget.count())):
                self.guiWindow.stackedWidget.currentWidget().tabWidget.setCurrentIndex(eachTabIndex)
                
                if level == 'beginner':
                    self.guiWindow.stackedWidget.currentWidget().tabWidget.currentWidget().\
                    springTab.hideEntriesBasedOnLevel(0)
        

    def setBackToPreviousView(self):
        self.guiWindow.stackedWidget.setCurrentIndex(self.currentIndex)
        self.guiWindow.stackedWidget.currentWidget().tabWidget.setCurrentIndex(self.currentTabIndex)


    def changeToBeginnerDisplay(self):
        self.setWindowTitle('Beginner')
        
        self.removeAllTabsFromAllWidgets()
        self.addAllExpertTabs()
        self.displayEntriesBasedOnLevel('beginner')
        self.removeTabsInThreeWidgetsFromIndices([6, 5, 4, 3, 1, 0], [7, 6, 5, 4, 3, 2, 1], [7, 6, 5, 4, 3, 1, 0])
        
        self.setBackToPreviousView()
        
        
    def changeToIntermediateDisplay(self):
        self.setWindowTitle('Intermediate')
    
        self.removeAllTabsFromAllWidgets()
        self.addAllExpertTabs()
        self.displayEntriesBasedOnLevel('intermediate')
        self.removeTabsInThreeWidgetsFromIndices([6, 5, 4, 3], [7, 6, 5, 3, 1], [7, 6, 4, 1, 0])
        
        self.setBackToPreviousView()
        
        
    def addAllExpertTabs(self):
        self.guiWindow.springMicrographW.addScanSpringTabs()
        self.guiWindow.spring2dW.addSpring2dTabs()
        self.guiWindow.spring3dW.addSpring3dTabs()


    def changeToExpertDisplay(self):
        self.setWindowTitle('Expert')
        
        self.removeAllTabsFromAllWidgets()
        self.addAllExpertTabs()
        self.displayEntriesBasedOnLevel('expert')
        
        self.setBackToPreviousView()
        

class AutoRefinementCycleTable(object):
    pass
 

class AutoRefinementCycleHelixTable(object):
    pass
 

class AutoRefinementCycleSegmentTable(object):
    pass
 

class AutoRefinementCycleSegmentSubunitTable(object):
    pass
 

class AutoSegmentTable(object):
    pass
 

class AutoHelixTable(object):
    pass
 

class AutoCtfMicrographTable(object):
    pass
 

class AutoCtfFindMicrographTable(object):
    pass
 

class AutoCtfTiltMicrographTable(object):
    pass
 

class AutoSubunitTable(object):
    pass


class AutoGridTable(object):
    pass
 

class AutoGridRefineTable(object):
    pass
 

class DataBaseUpdate(object):    
    def update_refinement_db(self, input_db, output_db):
        s = SpringDataBase()
        tables = ['cycles', 'cycle_helices', 'cycle_segments', 'cycle_subunits']
    
        autotables = [AutoRefinementCycleTable, AutoRefinementCycleHelixTable, AutoRefinementCycleSegmentTable,
        AutoRefinementCycleSegmentSubunitTable]
    
        target_tables = [RefinementCycleTable, RefinementCycleHelixTable, RefinementCycleSegmentTable,
        RefinementCycleSegmentSubunitTable]
    
        session = s.autoload_tables_into_session(input_db, tables, autotables)
        updated_session = s.setup_sqlite_db(refine_base, output_db)
    
        for each_autotable, each_target_table in zip(autotables, target_tables):
            updated_session = s.transfer_records_from_table(each_autotable, each_target_table, session, updated_session)
        updated_session.commit()
        

    def update_spring_db(self, input_db, output_db):
        s = SpringDataBase()
        tables = ['segments', 'helices', 'CtfMicrographs', 'CtfFindMicrographs', 'CtfTiltMicrographs', 'subunits']

        autotables = [AutoSegmentTable, AutoHelixTable, AutoCtfMicrographTable, AutoCtfFindMicrographTable,
        AutoCtfTiltMicrographTable, AutoSubunitTable]

        target_tables = [SegmentTable, HelixTable, CtfMicrographTable, CtfFindMicrographTable,CtfTiltMicrographTable,
        SubunitTable]
    
    
        session = s.autoload_tables_into_session(input_db, tables, autotables)
        updated_session = s.setup_sqlite_db(base, output_db)
    
        for each_autotable, each_target_table in zip(autotables, target_tables):
            updated_session = s.transfer_records_from_table(each_autotable, each_target_table, session, updated_session)
        updated_session.commit()


    def update_grid_db(self, input_db, output_db):
        s = SpringDataBase()
        tables = ['grids', 'grid_refine']
        autotables = [AutoGridTable, AutoGridRefineTable]
        target_tables = [GridTable, GridRefineTable]
    
        session = s.autoload_tables_into_session(input_db, tables, autotables)
        updated_session = s.setup_sqlite_db(grid_base, output_db)
    
        for each_autotable, each_target_table in zip(autotables, target_tables):
            updated_session = s.transfer_records_from_table(each_autotable, each_target_table, session, updated_session)
        updated_session.commit()


def define_program_options():
    proginfo = __doc__
    progname = 'emspring'
    eggmeta = GetMetaData(progname).release_meta()
    parser = ArgumentParser(prog=progname, description=proginfo)
    parser.add_argument('-inp', '--input', action='store', help='source database (*db) file name')
    parser.add_argument('-out', '--output', action='store', help='updated database (*db) file name')

    parser.add_argument('--version', action='version', version='GUI from package {0}-{1}'.format(progname.title(),
    eggmeta['Version']))

    parser.add_argument('-udb', '--update_db', action='store', help='Choose whether to update \'spring.db\', ' + \
    '\'refinement.db\' or \'grid.db\'.')

    args = parser.parse_args()

    return args


def main():
    args = define_program_options()
    
    if args.update_db is None:
        app = QApplication(sys.argv)
        window = SpringMain()

        window.show()
        app.exec_()

    if args.update_db in ['refinement.db', 'spring.db', 'grid.db'] and args.input is None and args.output is None: 
        print 'No databases were updated. Please provide input and output names for databases.'
    elif args.update_db in ['refinement.db']:
        DataBaseUpdate().update_refinement_db(args.input, args.output)
    elif args.update_db in ['spring.db']:
        DataBaseUpdate().update_spring_db(args.input, args.output)
    elif args.update_db in ['grid.db']:
        DataBaseUpdate().update_grid_db(args.input, args.output)
            

if __name__ == '__main__':
    main()




#!/usr/bin/env python
"""
Test module to check segmentplot
"""
from glob import glob
from spring.csinfrastr.csproductivity import TestingDirectory
from spring.segment2d.segmentplot import SegmentPlot, SegmentPlotPar
from spring.segment3d.refine.sr3d_main import SegmentRefine3d
from spring.tests.csinfrastr.test_csreadinput import EndToEndTest
from spring.tests.segment2d.test_segmentselect import TestSegmentSelect
from spring.tests.segment3d.test_segmentrefine3d import TestSegmentRefine3d
import numpy as np
import os


class TestSegmentPlotPrepare(object):
    def generate_input_database_from_helix(self):
        self.azimuthal_step = 20
        self.out_of_plane_range = [-8, 8]
        self.out_of_plane_step = 8
        self.helical_rotation = 50.0
        self.helix_start = 1
        
        self.euler_list = SegmentRefine3d().generate_Euler_angles_for_projection(self.azimuthal_step,
        self.out_of_plane_range, self.out_of_plane_step, self.helical_rotation)
        
        self.perturbed_alignment_parameters = \
        TestSegmentRefine3d().apply_random_rotations_x_and_y_shifts_to_projection_parameters(self.euler_list, 30)
        
        coord = np.arange(len(self.perturbed_alignment_parameters)) * 100.0
        coordinates = ((coord, coord))
        TestSegmentRefine3d().prepare_database_for_refinement(self.perturbed_alignment_parameters, 5.0, coordinates)
        

    def setup_selection_criteria_micrograph_and_helix(self, feature_set):
        feature_set.parameters['Micrographs select option'] = False
        feature_set.parameters['Include or exclude micrographs'] = 'include'
        feature_set.parameters['Micrographs list'] = '1'
        
        feature_set.parameters['Helices select option'] = False
        feature_set.parameters['Include or exclude helices'] = 'include'
        feature_set.parameters['Helices list'] = '1'
    
        return feature_set
        

class TestSegmentPlot(TestSegmentPlotPrepare, SegmentPlot):
    def setup(self):
        self.testingdir = TestingDirectory(os.path.splitext(os.path.basename(__file__))[0])
        self.testingdir.create()

        self.feature_set = SegmentPlotPar()
        self.feature_set.parameters['spring.db file']=os.pardir + os.sep + 'spring.db'
        self.feature_set.parameters['Diagnostic plot prefix'] = 'test_diagnostic.png'
        self.feature_set.parameters['Quantities'] = 'in-plane_rotation'#
        self.feature_set.parameters['Batch mode']=True#False
        self.feature_set.parameters['Set size']='helix'

        self.tsegsel = TestSegmentSelect()
        self.tsegsel.test_segment_file = 'test_segment_file.dat'
        self.feature_set = self.tsegsel.setup_selection_criteria_from_segment_table(self.feature_set)
        self.tsegsel.prepare_segment_file()
        
        self.generate_input_database_from_helix()
        
        super(TestSegmentPlot, self).__init__(self.feature_set)
        
        
    def teardown(self):
        os.remove('spring.db')
        os.remove(os.pardir + os.sep + 'spring.db')
        diagnostic_file = glob('test_diagnosti*.png')
        if self.batch_mode and diagnostic_file != []:
            os.remove(diagnostic_file[0])
        
        self.testingdir.remove()

        
class TestSegmentPlotMain(TestSegmentPlot):
    def do_test_case_sp1(self):
        self.plot_desired_quantities()
        

class TestSegmentPlotMore(TestSegmentPlot):
    def do_test_case_sp2(self):
        self.feature_set.parameters['Quantities'] = 'coordinates'#'in-plane_rotation'
        super(TestSegmentPlot, self).__init__(self.feature_set)
        
        self.plot_desired_quantities()
        
        
    def do_test_case_sp3(self):
        self.feature_set.parameters['Quantities'] = 'curvature'
        super(TestSegmentPlot, self).__init__(self.feature_set)
        
        self.plot_desired_quantities()
        
        
    def do_test_case_sp4(self):
        self.feature_set.parameters['Quantities'] = 'defocus'
        super(TestSegmentPlot, self).__init__(self.feature_set)
        
        self.plot_desired_quantities()
        
        
    def do_test_case_sp5(self):
        self.feature_set.parameters['Quantities'] = 'astigmatism'
        super(TestSegmentPlot, self).__init__(self.feature_set)
        
        self.plot_desired_quantities()
        
        
    def do_test_case_sp6(self):
        self.feature_set.parameters['Quantities'] = 'layer-line correlation'
        super(TestSegmentPlot, self).__init__(self.feature_set)
        
        self.plot_desired_quantities()
        
        
    def do_test_case_sp7(self):
        self.feature_set.parameters['Quantities'] = 'classes'
        super(TestSegmentPlot, self).__init__(self.feature_set)
        
        self.plot_desired_quantities()
        
        
    def do_test_case_sp8(self):
        self.feature_set.parameters['Set size'] = 'micrograph'
        super(TestSegmentPlot, self).__init__(self.feature_set)
         
        self.plot_desired_quantities()
        
        
    def do_test_case_sp9(self):
        self.feature_set.parameters['Set size'] = 'data_set'
        super(TestSegmentPlot, self).__init__(self.feature_set)
        
        self.plot_desired_quantities()
        
        
    def do_test_case_sp10(self):
        self.feature_set.parameters['Quantities'] = 'class_models'
        super(TestSegmentPlot, self).__init__(self.feature_set)
        
        self.plot_desired_quantities()


class TestSegmentPlotEndToEnd(TestSegmentPlot):
    def do_end_to_end_test_sp_inputfile(self):
        EndToEndTest().do_end_to_end_inputfile(self.feature_set)
 

    def do_end_to_end_test_sp_prompt(self):
        EndToEndTest().do_end_to_end_prompt(self.feature_set)
        
    
def main():
    tsp = TestSegmentPlotMain()
    tsp.setup()
    tsp.do_test_case_sp1()
        
if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""
Test module to check segclassexam
"""
from glob import glob
from multiprocessing import cpu_count
from spring.csinfrastr.csproductivity import TestingDirectory
from spring.segment2d.segclassexam import SegClassExam, SegClassExamPar
from spring.tests.csinfrastr.test_csreadinput import EndToEndTest
from spring.tests.segment2d.test_segmentexam import HelixProjectionPreparation
from statistics import add_series
import os


class HelixClassAvgPreparation(HelixProjectionPreparation):
    def prepare_helix_class_averages_and_variances(self):
        """
        * Function to generate a projection of one class
        """
        self.width_of_helix_in_angstrom = 200
        self.prepare_helix_projection()

        self.average =  'test_averages.hdf' 
        self.variance = 'test_variances.hdf'

        self.avg, self.var = add_series('test_segments.hdf')
        self.avg.append_image(self.average)
        self.var.append_image(self.variance)

class TestSegClassExam(HelixClassAvgPreparation, SegClassExam):
    def setup(self):
        self.testingdir = TestingDirectory(os.path.splitext(os.path.basename(__file__))[0])
        self.testingdir.create()

        self.feature_set = SegClassExamPar()
        self.prepare_helix_class_averages_and_variances()

        self.feature_set.parameters['Class average stack']=self.average
        self.feature_set.parameters['Diagnostic plot']='test_segclassexam.png'
        self.feature_set.parameters['Class variance option']=True
        self.feature_set.parameters['Class variance stack']=self.variance
        
        self.feature_set.parameters['Power spectrum output image']='test_power.hdf'
        self.feature_set.parameters['Enhanced power spectrum option']=True
        self.feature_set.parameters['Enhanced power spectrum output image']='test_power_enhanced.hdf'
        self.feature_set.parameters['Pixel size in Angstrom']=self.pixelsize
        self.feature_set.parameters['Estimated helix width in Angstrom']=int(1.2*self.width_of_helix_in_angstrom)
        self.feature_set.parameters['Power spectrum resolution cutoff in 1/Angstrom']=float(1/(self.pixelsize*2))
        self.feature_set.parameters['Class number range to be analyzed']=((0, 0))

        self.feature_set.parameters['MPI option']=False
        self.feature_set.parameters['Number of CPUs']=min(8, cpu_count())
        self.feature_set.parameters['Temporary directory']=os.path.abspath(os.curdir)

        super(TestSegClassExam, self).__init__(self.feature_set)

    def teardown(self):
        os.remove(self.average)
        os.remove(self.avgstack)
        os.remove(self.variance)
        if self.feature_set.parameters['Class variance option']:
            os.remove(self.varstack)
        diagnostic_files = glob('test_segclassexam*')
        for each_file in diagnostic_files:
            os.remove(each_file)
        os.remove('test_segments.hdf')
        os.remove('test_power.hdf')
        os.remove('test_power_enhanced.hdf')
        
        self.testingdir.remove()

        
class TestSegClassExamMain(TestSegClassExam):
    def do_test_case_sce1(self):
        self.exam_classes()


class TestSegClassExamMore(TestSegClassExam):
    def do_test_case_sce2(self):
        self.prepare_helix_class_averages_and_variances()
        self.prepare_helix_class_averages_and_variances()
        self.feature_set.parameters['Diagnostic plot']='test_segclassexam.pdf'
        self.feature_set.parameters['Class number range to be analyzed']=((0, 200))
        super(TestSegClassExam, self).__init__(self.feature_set)

        self.exam_classes()
        

    def do_test_case_sce3(self):
        self.feature_set.parameters['Class variance option']=False
        self.feature_set.parameters['Class variance stack']=self.variance

        super(TestSegClassExam, self).__init__(self.feature_set)

        self.exam_classes()


class TestSegClassExamEndToEnd(TestSegClassExam):
    def do_end_to_end_test_sce_inputfile(self):
        EndToEndTest().do_end_to_end_inputfile(self.feature_set)

    def do_end_to_end_test_sce_prompt(self):
        EndToEndTest().do_end_to_end_prompt(self.feature_set)


class TestSegClassExamMpi(TestSegClassExam):
    def do_end_to_end_test_sa_inputfile_with_MPI(self):
        self.prepare_helix_class_averages_and_variances()
        self.prepare_helix_class_averages_and_variances()
        self.feature_set.parameters['Diagnostic plot']='test_segclassexam.pdf'
        self.feature_set.parameters['Class number range to be analyzed']=((0, 2))
        self.feature_set.parameters['MPI option'] = True
        self.feature_set.parameters['Number of CPUs']=2
        
        EndToEndTest().do_end_to_end_inputfile(self.feature_set)
        
def main():
    tsce = TestSegClassExamMain()
    tsce.setup()
    tsce.do_test_case_sce1()
        
if __name__ == '__main__':
    main()



# Author: Carsten Sachse 29-Aug-2011
# Copyright: EMBL (2010 - 2018)
# License: see license.txt for details
"""
Program to interactively explore grid searches according to different criteria
"""
from EMAN2 import EMData
from PyQt4.QtCore import SIGNAL, Qt
from PyQt4.QtGui import QApplication, QWidget, QStackedWidget, QPushButton, QComboBox, QSplitter, QTableWidget, \
    QFileDialog, QGridLayout
from functools import partial
from spring.csinfrastr.csdatabase import SpringDataBase, GridRefineTable, GridTable, grid_base
from spring.csinfrastr.csfeatures import Features
from spring.csinfrastr.csgui import QTabWidgetCloseable
from spring.csinfrastr.cslogger import Logger
from spring.csinfrastr.csproductivity import ExtLauncher
from spring.csinfrastr.csreadinput import OptHandler
from spring.segment3d.segclassreconstruct import SegClassReconstruct
from spring.springgui.springdataexplore import SpringDataExplore, SpringDataExploreTablePane, SpringDataExploreDraw, \
    SpringDataExplore3d, SpringCommon
from sqlalchemy.sql.expression import desc, asc
import numpy as np
import os
import shutil
import sys

class SegGridExplorePar(object):
    """
    Class to initiate default dictionary with input parameters including help and range values and status dictionary
    """
    def __init__(self):
        # package/program identity
        self.package = 'emspring'       
        self.progname = 'seggridexplore'
        self.proginfo = __doc__
        self.code_files = [self.progname]

        self.seggridexplore_features = Features()
        self.feature_set = self.seggridexplore_features.setup(self)
        
        self.define_parameters_and_their_properties()
        self.define_program_states()


    def define_parameters_and_their_properties(self):
        self.feature_set = self.seggridexplore_features.set_grid_database(self.feature_set, singular=False)
        self.feature_set = self.set_subgrid_merge_option(self.feature_set)
        self.feature_set = self.set_reconstitute_grid(self.feature_set)
        
        self.feature_set = self.seggridexplore_features.set_interactive_vs_batch_mode(self.feature_set)
        self.feature_set = self.set_file_prefix_symmetry_grid(self.feature_set)
        self.feature_set = self.set_parameter_pair_to_be_extracted(self.feature_set)
        
    def define_program_states(self):
        self.feature_set.program_states['extract_desired_em_files']='Extract desired EM files'
        self.program_states['launch_seggridexplore_gui']='Explore different grid combinations interactively'

    
    def set_file_prefix_symmetry_grid(self, feature_set):
        inp9 = 'EM name'
        feature_set.parameters[inp9]='recvol.hdf'
        feature_set.properties[inp9]=feature_set.file_properties(1,['hdf'],'saveFile')
        feature_set.hints[inp9]='Template output name for associated EM files from grid search: accepted image file ' +\
        'formats ({0})'.format(', '.join(feature_set.properties[inp9].ext))
        
        feature_set.level[inp9]='beginner'
        feature_set.relatives[inp9]='Batch mode'
        
        return feature_set
    
    
    def set_parameter_pair_to_be_extracted(self, feature_set):
        inp9 = 'Parameter pair'
        feature_set.parameters[inp9]=((1.408, 22.03))
        feature_set.properties[inp9] = feature_set.Range(-1000, 1000, 0.001)
        feature_set.hints[inp9]='Parameter pair to extract associated EM files.'
        feature_set.relatives[inp9]=(('Batch mode', 'Batch mode'))
        feature_set.level[inp9]='intermediate'
        
        return feature_set
    
    
    def set_subgrid_merge_option(self, feature_set):
        inp8 = 'Subgrid merge option'
        feature_set.parameters[inp8] = False
        feature_set.hints[inp8] = 'Merge subgrids from multiple grid searches. Either these subgrids were ' + \
        'generated using the subgrid option in segrefine3drid or they originate from adjacent grids with the ' + \
        'same grid spacing. Enter subgrids using wildcards * or ?? can be used to specify multiple input files.'
        feature_set.level[inp8]='expert'
        
        return feature_set
    
    
    def set_reconstitute_grid(self, feature_set):
        inp1 = 'Reconstitute Grid'
        feature_set.parameters[inp1]=bool(True)
        feature_set.hints[inp1]='In case subgrids are merged reconstitute entire grid by reproducing directory tree ' + \
        'in one place and generate symbolic links to associated original files.'
        feature_set.relatives[inp1]='Subgrid merge option'
        feature_set.level[inp1]='expert'

        return feature_set
    
    
class SegGridExplorePreparation(object):
    def __init__(self, parset = None):
        self.log = Logger()
        if parset is not None:
            self.feature_set = parset
            p = self.feature_set.parameters
            
            self.grid_database = p['Grid database']
            self.infile = self.grid_database
            self.outfile = p['EM name']
            self.batch_mode = p['Batch mode']
            self.parameter_pair = p['Parameter pair']
            self.reconstitute_grid = p['Reconstitute Grid']
            
            self.subgrid_merge = p['Subgrid merge option']
            if self.subgrid_merge:
                self.subgrid_files = Features().convert_list_of_files_from_entry_string(self.grid_database)


class SegGridExploreSupport(object):
    def convert_None2nan(self, x):
        """
        Converts None objects to nan, for use in
        NumPy arrays. Returns an array.
        >>> from spring.segment3d.seggridexplore import SegGridExploreSupport
        >>> SegGridExploreSupport().convert_None2nan(5*[None])
        [nan, nan, nan, nan, nan]
        """
        newlist = []
        for i in x:
            if i is not None:
                newlist.append(i)
            else:
                newlist.append(np.nan)
             
        return newlist
    

    def get_general_grid_parameters(self, grid_file):
        grid_session = SpringDataBase().setup_sqlite_db(grid_base, grid_file)
        grid_combinations = grid_session.query(GridRefineTable).order_by(asc(GridRefineTable.id)).all()
        grid_parameters = grid_session.query(GridTable).filter(GridTable.id == grid_combinations[0].grid_id).first()
        
        return grid_session, grid_combinations, grid_parameters
    

    def get_rises_and_rotations_and_entered_symmetry_grid(self, grid_combinations):
        rise_or_pitches = np.array([each_grid_combination.primary_value for each_grid_combination in grid_combinations])
         
        rotation_or_unit_numbers = np.array([each_grid_combination.secondary_value for each_grid_combination in \
        grid_combinations])
         
        entered_symmetry_grid = SegClassReconstruct().convert_rise_rotation_or_pitch_unitnumber_series_to_grid_of_tuples\
        (np.unique(rise_or_pitches), np.unique(rotation_or_unit_numbers))
 
        return rise_or_pitches, rotation_or_unit_numbers, entered_symmetry_grid


    def get_all_attr_from_search_result(self, all_grid_combinations, db_attr, filtered_attr, grid_data, zlabels, table):
        list_of_val = [getattr(each_grid_combination, db_attr) for each_grid_combination in \
        all_grid_combinations]
        
        if set(list_of_val) != set([None]):
            conv_values = self.convert_None2nan(list_of_val)
            grid_data.append(conv_values)
            
            zlabels += SpringDataBase().get_labels_from_table(table, db_attr)
            filtered_attr += [db_attr]
            
        return grid_data, zlabels, filtered_attr


    def get_common_criterias(self):
        return ['fsc_0143', 'fsc_05', 'xshift_error', 'inplane_error', 'outofplane_error', 'mean_peak', 
                          'amp_corr_quarter_nyquist', 'amp_corr_half_nyquist', 'amp_corr_3quarter_nyquist', 
                          'amp_correlation', 'helical_ccc_error', 'mean_helical_ccc', 'out_of_plane_dev',
                          'variance', 'asym_unit_count', 'avg_azimuth_sampling', 'dev_azimuth_sampling']
                          
        
    def get_criterias(self):
        additional_criteria = ['phase_residual', 'diff_noise_ccc', 'diff_noise_amp_ccc', 'cross_correlation', 
        'diff_noise_pr','excluded_inplane_ratio', 'excluded_out_of_plane_ratio', 'excluded_total_ratio', 
        'noise_ccc', 'noise_amp_ccc', 'noise_pr', 'precision', 'recall', 'f1_measure', 'f05_measure']
         
        combined_criteria = self.get_common_criterias() + additional_criteria
        return combined_criteria
    
        
    def get_grid_information_from_database(self, grid_file):
        grid_session, grid_combinations, grid_parameters = self.get_general_grid_parameters(grid_file)
        
        rise_or_pitches, rotation_or_unit_numbers, grid_pairs = \
        self.get_rises_and_rotations_and_entered_symmetry_grid(grid_combinations)
        
        grid_data = []
        zlabels = []
        filtered_attr = []
        for each_attr in self.get_criterias():
            grid_data, zlabels, filtered_attr = self.get_all_attr_from_search_result(grid_combinations, each_attr,
            filtered_attr, grid_data, zlabels, GridRefineTable)
            
        if len(grid_data[0]) < (grid_pairs.shape[0] * grid_pairs.shape[1]):
            msg = 'The provided grid.db contains less data than expected from the grid parameters. ' + \
            'Double-check whether all grid runs finished successfully.'
            raise ValueError(msg)

        grid_data = [np.array(each_grid).reshape((grid_pairs.shape)) for each_grid in grid_data]
        
        files = [each_grid_combination.em_files_2d for each_grid_combination in grid_combinations]
        files_2d = [files for each_grid_data in grid_data]
        files = [each_grid_combination.em_files_3d for each_grid_combination in grid_combinations]
        files_3d = [files for each_grid_data in grid_data]
        
        last_grid = grid_session.query(GridTable).order_by(desc(GridTable.id)).first()
        xlabels = [last_grid.primary_variable for each_grid_data in grid_data]
        ylabels = [last_grid.secondary_variable for each_grid_data in grid_data]
        
        zxy_labels = zip(zlabels, xlabels, ylabels)
        grid_pairs = [grid_pairs for each_grid in grid_data]
        
        hints = SpringDataBase().get_hints_from_grid_table(filtered_attr)
        
        return grid_data, grid_pairs, zxy_labels, files_2d, files_3d, hints
        

class SegGridExploreGuiPane(QWidget):

    def __init__(self, grid, grid_pairs, zxy_labels, files_2d, files_3d, parent = None):
        QWidget.__init__(self, parent)
        
        self.tabWidget = QTabWidgetCloseable()

        self.grid = grid
        self.grid_pairs = grid_pairs
        self.zxy_labels = zxy_labels
        self.files_2d = files_2d
        self.files_3d = files_3d
        
        self.layout = QGridLayout()
        if self.grid.shape[0] == 1 or self.grid.shape[-1] == 1:
            self.add_1d_plot(((0, 0)), self.zxy_labels[1])
        else:
            self.setup_matrix_image_plot()
        
        self.table_matrix = self.prepare_table_matrix_from_symmetry_grid_pairs(self.grid, self.grid_pairs)
        self.table_pane = SpringDataExploreTablePane(self.table_matrix, ['GridID'] + list(self.zxy_labels)) 
        self.table_pane.data_table.primary_id_signal.connect(partial(self.launch_external_viewer_from_index))
        self.table_pane.data_table.setSelectionBehavior(QTableWidget.SelectRows)
        self.table_pane.data_table.setToolTip('Open connected data point by clicking on beginning of row.')

        self.splitter = QSplitter(Qt.Horizontal)
        self.splitter.addWidget(self.table_pane)
        self.splitter.addWidget(self.tabWidget)
        
        self.layout.addWidget(self.splitter, 0, 0, 2, 5)
        self.setLayout(self.layout)

        self.external_launcher = ExtLauncher()
        

    def setup_matrix_image_plot(self):
        self.seggridexplore2d_tab = SpringDataExplore()
        self.seggridexplore2d_tab.on_draw(np.array(self.grid), self.grid_pairs, self.zxy_labels, '2d')
        self.seggridexplore2d_tab.picked_left_point.connect(partial(self.launch_external_viewers_from_index_pair))
        self.seggridexplore2d_tab.picked_middle_point.connect(partial(self.add_additional_1d_tab))
        
        self.seggridexplore2d_tab.main_frame.setToolTip('Left mouse button to open associated reprojections and ' + \
        'power spectrum (and reconstruction if provided).\nMiddle mouse button to open 1D profile of associated ' + \
        '{rise_pitch} and {rot_number}.'.format(rise_pitch=self.zxy_labels[1], rot_number=self.zxy_labels[2]))
        
        self.tabWidget.addTab(self.seggridexplore2d_tab, '2D grid')
        self.button3d = QPushButton()
        self.button3d.setText('Explore3d')
        self.connect(self.button3d, SIGNAL('clicked()'), self.add_3d_surface_tab)
        self.pull_down3d = QComboBox()
        self.pull_down3d.addItems(['Visvis (fast)', 'Matplotlib (slow)'])
        
        self.pull_down3d.setToolTip('Choose between fast PyOpenGL (visvis) display with less navigation options ' + \
        'for large data grids or \nslow display with many navigation options for small data grids (matplotlib)')
        
        self.layout.addWidget(self.pull_down3d, 2, 3, 1, 1)
        self.layout.addWidget(self.button3d, 2, 4, 1, 1)


    def get_corresponding_symmetry_pair_from_picked_indices(self, index_pair):
        x_index, y_index = index_pair
        symmetry_pair = self.grid_pairs[int(y_index)][int(x_index)]
#        print index_pair, symmetry_pair
        
        return symmetry_pair, x_index, y_index


    def add_1d_plot(self, index_pair, symmetry_pair):
        self.seggridexplore1d_tab = SpringDataExplore()
        self.seggridexplore1d_tab.on_draw(self.grid, self.grid_pairs, self.zxy_labels, '1d', index_pair)
        
        self.tabWidget.addTab(self.seggridexplore1d_tab, '1D: ' + \
        '{0}'.format(symmetry_pair.__str__().strip(')').strip('(')))
        
        self.tabWidget.setCurrentIndex(self.tabWidget.indexOf(self.seggridexplore1d_tab))
        self.seggridexplore1d_tab.picked_left_point.connect(partial(self.launch_external_viewers_from1d))
        self.seggridexplore1d_tab.picked_label.connect(partial(self.define_labels_for_1d_profile))
        self.seggridexplore1d_tab.index_2d.connect(partial(self.define_2d_index_for_1d_profile))


    def add_additional_1d_tab(self, index_pair):
        symmetry_pair, x_index, y_index = self.get_corresponding_symmetry_pair_from_picked_indices(index_pair)
        self.add_1d_plot(index_pair, symmetry_pair)
        
        
    def add_3d_surface_tab(self):
        temp, helical_rotations = list(zip(*self.grid_pairs[0,].tolist()))
        helical_rises, temp = list(zip(*self.grid_pairs[:,0].tolist()))
        
        helical_rotations, helical_rises = np.meshgrid(helical_rotations, helical_rises)
        
        xyz_data = np.hstack([helical_rises.reshape(helical_rises.size, 1), 
                              helical_rotations.reshape(helical_rotations.size, 1), 
                              self.grid.reshape(self.grid.size, 1)])
        
        if str(str(self.pull_down3d.currentText())) == 'Visvis (fast)':
            self.seggridexplore3d_tab = SpringDataExplore3d(xyz_data, self.zxy_labels)
            self.tabWidget.addTab(self.seggridexplore3d_tab, '3D surface (vv)')
            
        elif str(str(self.pull_down3d.currentText())) == 'Matplotlib (slow)':
            self.seggridexplore3d_tab = SpringDataExplore()
            self.seggridexplore3d_tab.on_draw(self.grid, self.grid_pairs, self.zxy_labels, '3d')
            self.tabWidget.addTab(self.seggridexplore3d_tab, '3D surface (mpl)')
            
        self.tabWidget.setCurrentIndex(self.tabWidget.indexOf(self.seggridexplore3d_tab))
        
    
    def define_labels_for_1d_profile(self, label):
        self.label = label
#        print label
        
    def define_2d_index_for_1d_profile(self, index_pair):
        self.index_2dpair = index_pair
#        print index_pair
        
        
    def launch_external_viewers_from1d(self, picked_pair_from1d):
#        print(picked_pair_from1d)
        helical_rises, helical_rotations = \
        SpringDataExploreDraw().extract_rise_rotation_from_symmetry_grid(self.grid_pairs)
        
        if self.label == self.zxy_labels[1]:
            previous_rotation_index = self.index_2dpair[0]
            adjusted_rise = picked_pair_from1d[0] 
            
            matched_indices = np.where(np.array(helical_rises) == adjusted_rise)
            
            y_index = matched_indices[0][0]
            x_index = int(previous_rotation_index)
        
        elif self.label == self.zxy_labels[2]:
            previous_rise_index = self.index_2dpair[1]
            adjusted_rotation = picked_pair_from1d[0] 
            
            matched_indices = np.where(np.array(helical_rotations) == adjusted_rotation)
            
            y_index = int(previous_rise_index)
            x_index = matched_indices[0][0]
        
        stack_index = self.get_stack_index_from_rise_and_rotation_index(x_index, y_index)
        
        self.launch_external_viewers(stack_index)
        
        
    def launch_external_viewers(self, index):
        if self.files_2d[index] is not None:
            img = EMData()
                
            if not hasattr(self, 'opened'):
                self.opened = []
            else:
                [os.remove(each_opened) for each_opened in self.opened]
                self.opened = []
            
            for (each_file, stack_id) in self.files_2d[index]:
                if each_file.endswith('hdf'):
                    print each_file, stack_id
                    img.read_image(each_file, stack_id)
                    file_name = '{0}_{1:05}{2}'.format(os.path.splitext(each_file)[0], stack_id, os.path.splitext(each_file)[1])
                    img.write_image(file_name)
                    self.external_launcher.qlaunch_open_file(os.path.abspath(file_name))
                    self.opened.append(file_name)
                else:
                    self.external_launcher.qlaunch_open_file(each_file)
        
        if self.files_3d[index] is not None:
            self.external_launcher.qlaunch_open_file(self.files_3d[index][0])
            
    
    def launch_external_viewer_from_index(self, stack_index):
        self.launch_external_viewers(stack_index)


    def launch_external_viewers_from_index_pair(self, index_pair):
        x_index, y_index = index_pair
        stack_index = self.get_stack_index_from_rise_and_rotation_index(x_index, y_index)
        
        self.launch_external_viewers(stack_index)
        
        
    def get_stack_index_from_rise_and_rotation_index(self, x_index, y_index):
        x_count, y_count = self.grid_pairs.shape
        stack_index = int(x_index + y_count * y_index)
        
        return stack_index
    
    
    def exitApp(self):
        self.close()


    def prepare_table_matrix_from_symmetry_grid_pairs(self, symmetry_grid_np, symmetry_grid_pairs):
    
        sym_id = np.arange(symmetry_grid_np.size)
        symmetry_grid_cc_seq = symmetry_grid_np.ravel()
        rises_seq, rotation_seq = list(zip(*symmetry_grid_pairs.ravel()))
        
        table_matrix = np.vstack([sym_id, symmetry_grid_cc_seq, rises_seq, rotation_seq])
        
        return table_matrix
        

class SegGridExploreGui(SegGridExploreSupport, QWidget):
    def __init__(self, feature_set, parent = None):
        QWidget.__init__(self, parent)
        
        self.feature_set = feature_set
        self.properties = feature_set.properties
#        self.grid_database = self.feature_set.parameters['Grid database']
        self.grid_database = 'grid.db'
        
        self = SpringCommon().setup_spring_page_top(self, feature_set)
        
        self.grids, self.grid_pairs, self.zxy_labels, self.files_2d, self.files_3d, hints = \
        self.get_grid_information_from_database(self.grid_database)
        
        self.stackedPane = QStackedWidget()
        self.currentPaneIndex = 0
        self.panes = []
        for pane_id, each_grid in enumerate(self.grids):
            self.pane = SegGridExploreGuiPane(self.grids[pane_id], self.grid_pairs[pane_id], self.zxy_labels[pane_id],
            self.files_2d[pane_id], self.files_3d[pane_id])
            
            self.stackedPane.addWidget(self.pane)
            self.panes.append(self.pane)
        self.layout.addWidget(self.stackedPane, 1, 0, 2, 5)
        self.stackedPane.setCurrentIndex(self.currentPaneIndex)
        
        self.z_labels = [each_label[0] for each_label in self.zxy_labels ]

        self.pull_down_panes = QComboBox() 
        self.pull_down_panes.addItems(self.z_labels)
        self.pull_down_panes.setToolTip(hints)
        self.connect(self.pull_down_panes, SIGNAL('currentIndexChanged(int)'), self.update_panes)
        self.layout.addWidget(self.pull_down_panes, 0, 3, 1, 1)
        
        self.pull_down_pairs = QComboBox()
        self.items = []
        for each_grid in self.grid_pairs:
            self.items.append([ 'Grid ID:{0}: {1}'.format(each_grid_id, str(each_pair).strip(')').strip('(')) for \
            each_grid_id, each_pair in enumerate(each_grid.ravel())])
            
        self.pull_down_pairs.addItems(self.items[self.currentPaneIndex])
        self.layout.addWidget(self.pull_down_pairs, 3, 3, 1, 1)
        
        self.save_button = QPushButton()
        self.save_button.setText('Save')
        self.save_button.setToolTip('Save associated EM files.')
        self.save_button.setShortcut('Ctrl+S')
        self.connect(self.save_button, SIGNAL('clicked()'), self.saveEmFile)
        
        self.layout.addWidget(self.save_button, 3, 4, 1, 1)
        
        self.setLayout(self.layout)
        
        
    def update_panes(self, value):
        self.currentPaneIndex = value
        self.stackedPane.setCurrentIndex(self.currentPaneIndex)
        
        
    def saveEmFile(self):
        fname = str(QFileDialog.getSaveFileName(self , 'Save EM file', '.', 'Files ({0})'.\
        format('*.' + ' *.'.join(self.properties.values()[1].ext)))) 
        
        index = self.pull_down_pairs.currentIndex()
        files = self.files_2d[self.currentPaneIndex][index]
        img = EMData()
        for each_file_id, (each_file_name, each_stack_id) in enumerate(files):
            img.read_image(each_file_name, each_stack_id)
            img.write_image(os.path.splitext(fname)[0] + '2d' + os.path.splitext(fname)[-1], each_file_id)
            
        volume_name = self.files_3d[self.currentPaneIndex][index][0]
        if volume_name is not None:
            img.read_image(volume_name)
            img.write_image(os.path.splitext(fname)[0] + '3d' + os.path.splitext(fname)[-1])
            
        
class SegGridExploreMerge(SegGridExplorePreparation):

    def perform_simple_additive_merge(self, sorted_subgrid_files):
        merged_session = SpringDataBase().setup_sqlite_db(grid_base, 'grid.db')
        for each_subgrid_id, each_subgrid_file in enumerate(sorted_subgrid_files):
            each_grid = SpringDataBase().setup_sqlite_db(grid_base, each_subgrid_file)
            if each_subgrid_id == 0:
                merged_session = SpringDataBase().copy_all_table_data_from_one_session_to_another_session(GridTable,
                merged_session, each_grid, merge_ids=True)

            merged_session = SpringDataBase().copy_all_table_data_from_one_session_to_another_session(GridRefineTable,
            merged_session, each_grid, merge_ids=True)
        
        merged_session.commit()
        
        return merged_session
    

    def check_whether_seamless_grid_merging_is_possible(self, primary_min, primary_max, primary_inc):
        """
        >>> from spring.segment3d.seggridexplore import SegGridExplore
        >>> s = SegGridExplore()
        >>> s.check_whether_seamless_grid_merging_is_possible([16.25, 16.36], [16.35, 16.45], [0.01])
        >>> s.check_whether_seamless_grid_merging_is_possible([16.25, 16.38], [16.35, 16.45], [0.01])
        Traceback (most recent call last):
          File '/System/Library/Frameworks/Python.framework/Versions/2.7/lib/python2.7/doctest.py', line 1254, in __run
            compileflags, 1) in test.globs
          File '<doctest spring.segment3d.seggridexplore.SegGridExploreMerge.check_whether_seamless_grid_merging_is_possible[3]>', line 1, in <module>
            s.check_whether_seamless_grid_merging_is_possible([16.25, 16.38], [16.35, 16.45], [0.01])
          File '/Users/sachse/Documents/en_casa/EMBL/08_python/26_springbox/develop-eggs/spring-dev/spring/segment3d/seggridexplore.py', line 517, in check_whether_seamless_grid_merging_is_possible
            raise ValueError(msg)
        ValueError: You have specified grid datbases to be merged that can not be merged seamlessly without gaps into a continuous grid.

        """
        primary_min = [abs(each_min) for each_min in list(primary_min)]
        primary_max = [abs(each_max) for each_max in list(primary_max)]
        primary_inc = [abs(each_inc) for each_inc in  list(primary_inc)]
        
        primary_min.sort()
        primary_max.sort()
        for each_ext_id, each_max in enumerate(primary_max[:-1]):
            next_min = primary_min[each_ext_id + 1]
            computed_next_min = each_max + primary_inc[0]
            if abs(computed_next_min - next_min) > primary_inc[0]/100.0:
                msg = 'You have specified grid datbases to be merged that can not be merged seamlessly without ' + \
                'gaps into a continuous grid.'
                
                raise ValueError(msg)
            

    def update_new_grid_min_and_max_after_merge(self, primary_min, primary_max, second_min, second_max, merged_session):
        grid = merged_session.query(GridTable).first()
        grid.primary_min = min(primary_min)
        grid.primary_max = max(primary_max)
        grid.second_min = min(second_min)
        grid.second_max = max(second_max)
        merged_session.merge(grid)
        
        merged_session.commit()
        
        return merged_session
    

    def merge_grids_from_files(self, sorted_subgrid_files, primary_min, primary_max, second_min, second_max,
    primary_inc, second_inc, same_increments):
    
        same_max_and_mins = len(primary_min) + len(primary_max) + len(second_min) + len(second_max)
        if same_max_and_mins == 4 and same_increments == 2:
            merged_session = self.perform_simple_additive_merge(sorted_subgrid_files)
        if not same_max_and_mins == 4 and same_increments == 2:
            if 0 in second_inc or 0 in primary_inc:
                pass
            else:
                self.check_whether_seamless_grid_merging_is_possible(primary_min, primary_max, primary_inc)
                self.check_whether_seamless_grid_merging_is_possible(second_min, second_max, second_inc)
            merged_session = self.perform_simple_additive_merge(sorted_subgrid_files)
            merged_session = self.update_new_grid_min_and_max_after_merge(primary_min, primary_max, second_min, 
                second_max, merged_session)
            
            os.rename('grid.db', 'grid_not_sorted.db')
            tmp_session = SpringDataBase().setup_sqlite_db(grid_base, 'grid_not_sorted.db')
            
            grid = tmp_session.query(GridRefineTable).\
            order_by(GridRefineTable.primary_value.asc()).\
            order_by(GridRefineTable.secondary_value.asc()).all()
            
            shutil.copy('grid_not_sorted.db', 'grid.db')
            merged_session = SpringDataBase().setup_sqlite_db(grid_base, 'grid.db')
            columns = SpringDataBase().get_columns_from_table(GridRefineTable)
            for each_grid_id, each_grid_point in enumerate(grid):
                data = SpringDataBase().get_data_from_entry(columns, each_grid_point)
                data['id'] = each_grid_id + 1
                merged_session.merge(GridRefineTable(**data))
            
            merged_session.commit()
            os.remove('grid_not_sorted.db')
            
        return merged_session
    

    def merge_subgrids_into_single_grid_file(self, subgrid_files):
        split_files = [os.path.split(each_file)[-1] for each_file in subgrid_files]
        sorted_subgrid_files = [subgrid_files[each_ind] for each_ind in np.argsort(split_files)]
        
        file_grid_parameters = []
        
        for each_subgrid_file in sorted_subgrid_files:
            each_grid = SpringDataBase().setup_sqlite_db(grid_base, each_subgrid_file)
            grid_entries = each_grid.query(GridTable).first()
            file_grid_parameters.append(grid_entries)
        
        primary_var = set([each_file_grid.primary_variable for each_file_grid in file_grid_parameters])
        secondary_var = set([each_file_grid.secondary_variable for each_file_grid in file_grid_parameters])
        primary_min = set([each_file_grid.primary_min for each_file_grid in file_grid_parameters])
        primary_max = set([each_file_grid.primary_max for each_file_grid in file_grid_parameters])
        second_min = set([each_file_grid.second_min for each_file_grid in file_grid_parameters])
        second_max = set([each_file_grid.second_max for each_file_grid in file_grid_parameters])
        primary_inc = set([each_file_grid.primary_inc for each_file_grid in file_grid_parameters])
        second_inc = set([each_file_grid.second_inc for each_file_grid in file_grid_parameters])
            
        same_variables = len(primary_var) + len(secondary_var)
        same_increments = len(primary_inc) + len(second_inc)
        for each_inc in list(primary_inc) + list(second_inc):
            if each_inc == 0:
                same_variables = 2
                same_increments = 2
        
        entities = []
        if not same_variables == 2:
            entities.append('variables')
        if not same_increments == 2:
            entities.append('increments')
            
        if not same_variables == 2 or not same_increments == 2:
            msg = ''
            for each_entity in entities:
                msg += 'You have specified grid databases to be merged that do not match in their primary or ' + \
                'secondary {0}. This way they cannot be merged. Double-check the specified files. '.format(each_entity)
            raise ValueError(msg)
        
        merged_session = self.merge_grids_from_files(sorted_subgrid_files, primary_min, primary_max, second_min,
        second_max, primary_inc, second_inc, same_increments)
             
        return merged_session
    
    
    def write_symbolic_links_to_have_merged_data_in_one_place(self, grid_session):
        grid_points = grid_session.query(GridRefineTable).all()
        
        for each_grid_point in grid_points:
            for each_entry in each_grid_point.em_files_2d + each_grid_point.em_files_3d: 
                if type(each_entry) is tuple:
                    each_file = each_entry[0]
                else:
                    each_file = each_entry 
                    
                local_dir = os.path.split(os.path.dirname(each_file))[-1]
                if not os.path.isdir(local_dir):
                    os.mkdir(local_dir)
                os.chdir(local_dir)
                os.symlink(os.path.relpath(each_file), os.path.basename(each_file))
                os.chdir(os.pardir)
        
    
class SegGridExplore(SegGridExploreMerge):
    def launch_seggridexplore_gui(self, feature_set):
        self.log.fcttolog()
        app = QApplication(sys.argv)
        gridexplor = SegGridExploreGui(feature_set)
        gridexplor.show()
        app.exec_()

    
    def extract_desired_em_files(self):
        self.log.fcttolog()
        
        grids, grid_pairs, zxy_labels, files_2d, files_3d, hints = \
        SegGridExploreSupport().get_grid_information_from_database(self.grid_database)
        
        vol = EMData()
        for each_index, each_grid_pair in enumerate(grid_pairs[0].ravel()):
            if each_grid_pair == self.parameter_pair:
                vol.read_image(files_3d[0][each_index][0])
                
                outfile = '{0}{1:04}{2}'.format(os.path.splitext(self.outfile)[0], each_index,
                os.path.splitext(self.outfile)[-1])
                
                vol.write_image(outfile)
            self.log.plog(90 * (each_index + 1) / len(grid_pairs[0].ravel()) + 10)
                
                
    def launch_seggridexplore(self):
        if self.subgrid_merge:
            grid_session = self.merge_subgrids_into_single_grid_file(self.subgrid_files)
            if self.reconstitute_grid:
                self.write_symbolic_links_to_have_merged_data_in_one_place(grid_session)
        else:
            shutil.copy(self.grid_database, 'grid.db')
            
        self.log.plog(10)
        if not self.batch_mode:
            self.launch_seggridexplore_gui(self.feature_set)
        else:
            self.extract_desired_em_files()
        
        self.log.endlog(self.feature_set)
        
def main():
    # Option handling
    parset = SegGridExplorePar()
    mergeparset = OptHandler(parset)

    ######## Program
    sseg = SegGridExplore(mergeparset)
    sseg.launch_seggridexplore()

if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""
Test module to check new particlealign2d module
"""
from multiprocessing import cpu_count
from spring.csinfrastr.csproductivity import TestingDirectory
from spring.particle2d.particlealign2d import ParticleAlign2d, ParticleAlign2dPar
from spring.tests.csinfrastr.test_csreadinput import EndToEndTest
from spring.tests.segment2d.test_segmentalign2d import TestSegmentAlign2d, \
    TestSegmentAlign2dPreparation
import os


class TestParticleAlign2dPreparation(object):
    def setup_helix_or_particle_dimension(self):
        self.feature_set.parameters['Estimated inner and outer particle diameter in Angstrom'] = ((200, 750))

        
class TestParticleAlign2d(TestParticleAlign2dPreparation, TestSegmentAlign2dPreparation, ParticleAlign2d):
    def setup(self):
        self.testingdir = TestingDirectory(os.path.splitext(os.path.basename(__file__))[0])
        self.testingdir.create()

        self.feature_set = ParticleAlign2dPar()
        
        self.setup_of_common_non_mpi_parameters()
        self.feature_set.parameters['MPI option'] = False
        
        super(TestParticleAlign2d, self).__init__(self.feature_set)
        
    def do_test_case_pa1(self):

        self.perform_segmentalign2d()

        TestSegmentAlign2d().check_rotation(self.helix_stack, self.applied_rotations, self.delta_in_plane_rotation,
        self.rotation_step, self.restrain_option)
        
    def teardown(self):
        self.teardown_segmentalign2d()
        self.testingdir.remove()

        
class TestParticleAlign2dMore(TestParticleAlign2d):
    def do_test_case_pa2(self):
        self.restrain_option=False
        self.feature_set.parameters['Limit in-plane rotation']=self.restrain_option
        
        super(TestParticleAlign2d, self).__init__(self.feature_set)
        
        self.perform_segmentalign2d()

        TestSegmentAlign2d().check_rotation(self.helix_stack, self.applied_rotations, self.delta_in_plane_rotation,
        self.rotation_step, self.restrain_option)
        
    def do_end_to_end_test_sa_inputfile(self):
        EndToEndTest().do_end_to_end_inputfile(self.feature_set)
 
    def do_end_to_end_test_sa_prompt(self):
        EndToEndTest().do_end_to_end_prompt(self.feature_set)
        
class TestParticleAlign2dMpi(TestParticleAlign2d):
    def do_end_to_end_test_sa_inputfile_with_MPI(self):
        self.feature_set.parameters['MPI option'] = True
        self.feature_set.parameters['Number of CPUs']=min(8, cpu_count())
        
        EndToEndTest().do_end_to_end_inputfile(self.feature_set)

def main():
    talign = TestParticleAlign2d()
    talign.setup()
    talign.do_test_case_pa1()
        
if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""
Test module to check micctfdetermine
"""
from glob import glob
from multiprocessing import cpu_count
from spring.csinfrastr.csproductivity import TestingDirectory
from spring.micprgs.micctfdetermine import MicCtfDetermine, MicCtfDeterminePar
from spring.segment2d.segmentctfapply import SegmentCtfApply
from spring.tests.csinfrastr.test_csreadinput import EndToEndTest
from spring.tests.segment2d.test_segment import MicrographHelixPreparation
import numpy as np
import os
import shutil
import sys


class TestMicCtfDeterminePreparation(object):

    def setup_of_common_non_mpi_parameters(self):
        self.feature_set = MicCtfDeterminePar()
        self.pixelsize = 5.0
        self.avg_defocus=40000
        self.voltage=200
        self.cs=2.0
        self.ampcont=0.1
        self.astigmatism=5000
        self.astigmatism_angle=33.0
        
        self.micrograph_file = 'test_mic.hdf'
        self.diagnostic_output = 'test_output.pdf'
        
        MicrographHelixPreparation().prepare_ctf_micrograph(self.micrograph_file, self.avg_defocus, self.cs,
        self.voltage, self.pixelsize, self.ampcont, self.astigmatism, self.astigmatism_angle)

        self.feature_set.parameters['Micrographs']='test_mic.hdf'
        self.feature_set.parameters['Diagnostic plot pattern']=self.diagnostic_output
        
        self.feature_set.parameters['Spring database option']=False
        self.feature_set.parameters['spring.db file']=os.pardir + os.sep + 'spring.db'
        self.feature_set.parameters['Continue spring.db option']=False
            
        self.pardir = os.path.pardir
            
        self.feature_set.parameters['Pixel size in Angstrom'] = self.pixelsize
        self.feature_set.parameters['Spherical aberration'] = self.cs
        self.feature_set.parameters['Electron voltage in kV'] = self.voltage
        self.feature_set.parameters['Amplitude contrast'] = self.ampcont
        self.feature_set.parameters['Tile size power spectrum in Angstrom'] = 640
        self.feature_set.parameters['Range of defocus in Angstrom'] =((self.avg_defocus - 5000, self.avg_defocus + 5000))
        self.feature_set.parameters['Defocus search step size'] = 500
        self.feature_set.parameters['Astigmatism search restraint in Angstrom']=6000
        self.feature_set.parameters['Resolution search range in Angstrom']= ((30.0, 10.0))
        self.feature_set.parameters['CTFTILT refine option']=True
        self.feature_set.parameters['Local defocus search range'] = 5000
        self.feature_set.parameters['Expected tilt and tilt search range in degrees']= ((0, 10))

        self.feature_set.parameters['Binning option'] = False
        self.feature_set.parameters['Binning factor'] = 1
        self.feature_set.parameters['MPI option']=False
        self.feature_set.parameters['Number of CPUs']=min(8, cpu_count())
            
        self.feature_set.parameters['Temporary directory']=os.path.abspath(os.curdir)
        
class TestMicCtfDetermine(TestMicCtfDeterminePreparation, MicCtfDetermine):

    def setup(self):
        self.testingdir = TestingDirectory(os.path.splitext(os.path.basename(__file__))[0])
        self.testingdir.create()
        self.setup_of_common_non_mpi_parameters()
        super(TestMicCtfDetermine, self).__init__(self.feature_set)
        

    def check_avg_defocus(self):
        defocus1 = self.ctftilt_parameters.defocus1
        defocus2 = self.ctftilt_parameters.defocus2
        determined_defocus = round(defocus1 + defocus2) / 2
        sys.stderr.write('\n{avg_def} = {det_def} '.format(avg_def=self.avg_defocus, det_def=determined_defocus))
        assert self.avg_defocus == round(determined_defocus, -3)
        return defocus1, defocus2


    def check_astigmatism_and_astigmation_angle(self):
        defocus1 = self.ctftilt_parameters.defocus1
        defocus2 = self.ctftilt_parameters.defocus2
        
        astigmation_angle = self.ctftilt_parameters.astigmation_angle
        
        determined_defocus, determined_astigmatism, determined_astig_angle = \
        SegmentCtfApply().convert_mrc_defocus_to_sparx_defocus(defocus1, defocus2, astigmation_angle)
        
        sys.stderr.write('\n{avg_def} = {det_def} '.format(avg_def=self.avg_defocus, det_def=determined_defocus))
        assert self.avg_defocus == round(determined_defocus, -3)
        sys.stderr.write('\n{astig} = {det_astig} '.format(astig=self.astigmatism, det_astig=determined_astigmatism))
        assert self.astigmatism == round(determined_astigmatism, -3)
        
        sys.stderr.write('\n{a_angle} = {det_a_angle} '.format(a_angle=self.astigmatism_angle,
        det_a_angle=determined_astig_angle))
        
        assert self.astigmatism_angle == round(determined_astig_angle, -1)


    def check_tilt_angle(self):
        determined_tilt_angle = self.ctftilt_parameters.tilt_angle
        
        sys.stderr.write('\n{exp_angle} = {tilt_angle} '.format(exp_angle=self.expected_tilt_angle,
        tilt_angle=determined_tilt_angle))
        
        assert self.expected_tilt_angle == round(determined_tilt_angle)

    def check_that_parameters_were_correctly_detemined(self):
        defocus1, defocus2 = self.check_avg_defocus()
        self.check_astigmatism_and_astigmation_angle(defocus1, defocus2)
        self.check_tilt_angle()

    def teardown(self):
        database_file = 'spring.db'
        assert os.path.isfile(database_file)
        os.remove(database_file)
        os.remove(self.micrograph_file)
          
        logfiles = glob('test*{sep}log'.format(sep=os.extsep))
        diag_files = glob('test*{sep}pdf'.format(sep=os.extsep))
        for each_file in logfiles + diag_files:
            os.remove(each_file)
 
        self.testingdir.remove()


class TestMicCtfDetermineMain(TestMicCtfDetermine):
    def do_test_case_scd1(self):
        self.determine_ctf()


class TestMicCtfDetermineMore(TestMicCtfDetermine):
    def do_test_case_scd2(self):
        self.feature_set.parameters['CTFTILT refine option']=False
        super(TestMicCtfDetermine, self).__init__(self.feature_set)
            
        self.determine_ctf()
    
    def do_test_case_scd3(self):
        astigmation_angles = np.linspace(20, 160, 8)
        for each_angle in astigmation_angles:
            self.astigmatism_angle = each_angle
            
            MicrographHelixPreparation().prepare_ctf_micrograph(self.micrograph_file, self.avg_defocus, self.cs,
            self.voltage, self.pixelsize, self.ampcont, self.astigmatism, self.astigmatism_angle)
            
            if os.path.exists('spring.db'):
                os.remove('spring.db')
            self.determine_ctf()
            self.check_astigmatism_and_astigmation_angle()

        
class TestMicCtfDetermineEndtoEnd(TestMicCtfDetermine):
    def do_end_to_end_test_scd_inputfile(self):
        EndToEndTest().do_end_to_end_inputfile(self.feature_set)
        self.pardir = os.path.curdir
 
    def do_end_to_end_test_scd_prompt(self):
        EndToEndTest().do_end_to_end_prompt(self.feature_set)
        self.pardir = os.path.curdir
        
class TestMicCtfDetermineMpi(TestMicCtfDetermine):
    def do_end_to_end_test_sa_inputfile_with_MPI(self):
        self.feature_set.parameters['MPI option'] = True
        self.feature_set.parameters['Number of CPUs']=min(8, cpu_count())
        self.feature_set.parameters['Micrographs']='test_mi?.hdf'
        shutil.copy(self.micrograph_file, 'test_mi1.hdf')
        
        EndToEndTest().do_end_to_end_inputfile(self.feature_set)
        os.remove('test_mi1.hdf')
        self.pardir = os.path.curdir

    def do_end_to_end_test_sa_inputfile_no_CTFTILT_with_MPI(self):
        self.feature_set.parameters['MPI option'] = True
        self.feature_set.parameters['Number of CPUs']=min(8, cpu_count())
        self.feature_set.parameters['CTFTILT refine option']=False
        
        EndToEndTest().do_end_to_end_inputfile(self.feature_set)
        self.pardir = os.path.curdir
        
def main():
    trc = TestMicCtfDetermineMain()
    trc.setup()
    trc.do_test_case_scd1()
        
if __name__ == '__main__':
    main()



# Author: Carsten Sachse 21-Sep-2010
# Copyright: EMBL (2010 - 2018)
# License: see license.txt for details

""" 
Program to evaluate performance of scanner by measuring dots spaced 2.5 mm apart. 
"""
from EMAN2 import Util
from spring.csinfrastr.csfeatures import Features
from spring.csinfrastr.cslogger import Logger
from spring.csinfrastr.csproductivity import DiagnosticPlot
from spring.csinfrastr.csreadinput import OptHandler
from spring.micprgs.scansplit import Micrograph
import numpy as np

class ScanDotFitPar:
    """
    Class to initiate default dictionary with input parameters including help and range values and 
    status dictionary
    """
    def __init__(self):
        # package/program identity
        self.package = 'emspring'
        self.progname = 'scandotfit'
        self.proginfo = __doc__
        self.code_files = [self.progname]

        self.scandotfit_features = Features()
        self.feature_set = self.scandotfit_features.setup(self)
        
        self.define_parameters_and_their_properties()
        self.define_program_states()
        
    def define_parameters_and_their_properties(self):
        self.feature_set = self.scandotfit_features.set_inp_micrograph(self.feature_set)
        self.feature_set = self.scandotfit_features.set_output_plot(self.feature_set, self.progname + '_diag.pdf')
    
        self.feature_set = self.scandotfit_features.set_scanner_step_size(self.feature_set)
        self.feature_set = self.set_integration_width(self.feature_set)
        self.feature_set = self.set_topleft_coordinates(self.feature_set)
        self.feature_set = self.set_topright_coordinates(self.feature_set)
        self.feature_set = self.set_bottomleft_coordinates(self.feature_set)
        
    
    def define_program_states(self):
        self.feature_set.program_states['readmic']='Read input micrograph'
        self.feature_set.program_states['searchdots']='Determines positions of dots by center of gravity calculation'
        self.feature_set.program_states['dotdistance']='Calculates distance between dots'
        self.feature_set.program_states['visdots']='Visualization of dots'


    def set_integration_width(self, feature_set):
        inp4 = 'Width of integration in pixels'
        feature_set.parameters[inp4] = int(100)
        
        feature_set.hints[inp4] = 'Use default value: width of box in pixels used for dot determination - otherwise ' +\
        'use with caution.'
        
        feature_set.properties[inp4] = feature_set.Range(1, 500, 1)
        feature_set.level[inp4] = 'expert'
        
        return feature_set


    def set_topleft_coordinates(self, feature_set):
        inp5 = 'Topleft coordinates of dot grid' #p[inp5]=tuple((3309,13355))
        feature_set.parameters[inp5] = tuple((539, 2129))
        feature_set.hints[inp5] = 'Comma-separated integer pair of coordinates (x, y).'
        feature_set.properties[inp5] = self.feature_set.Range(0, 30000, 1)
        feature_set.level[inp5]='intermediate'
        
        return feature_set


    def set_topright_coordinates(self, feature_set):
        inp6 = 'Topright coordinates of dot grid' #p[inp6]=tuple((9023,13607))
        feature_set.parameters[inp6] = tuple((1493, 2173))
        feature_set.hints[inp6] = 'Comma-separated integer pair of coordinates (x, y).'
        feature_set.properties[inp6] = feature_set.Range(0, 30000, 1)
        feature_set.level[inp6]='intermediate'
        
        return feature_set


    def set_bottomleft_coordinates(self, feature_set):
        inp7 = 'Bottomleft coordinates of dot grid' #p[inp7]=tuple((3909,138))
        feature_set.parameters[inp7] = tuple((638, 46))
        feature_set.hints[inp7] = 'Comma-separated integer pair of coordinates (x, y).'
        feature_set.properties[inp7] = feature_set.Range(0, 30000, 1)
        feature_set.level[inp7]='intermediate'
        
        return feature_set


class ScanDotFit(object):
    """
    * Class that holds functions for automated distance measurements from regular dots on micrograph

    * __init__ Function to read in the entered parameter dictionary and load micrograph

    #. Usage: ScanDotFit(pardict)
    #. Input: pardict = OrderedDict of program parameters
    """
    def __init__(self, parset = None):
        self.log = Logger()
        if parset is not None:
            self.feature_set = parset  
            p = self.feature_set.parameters
            self.infile = p['Micrograph']
            self.outfile = p['Diagnostic plot']
            self.stepsize = p['Scanner step size in micrometer']
            self.boxwidth = p['Width of integration in pixels']
            self.topleft = p['Topleft coordinates of dot grid']
            self.topright = p['Topright coordinates of dot grid']
            self.bottomleft = p['Bottomleft coordinates of dot grid']

            #Micrograph.readmic(self, self.infile)
            self.img, self.nx, self.ny = Micrograph().readmic(self.infile)

    def pttoline(self, ptone = None, pttwo = None):
        """
        * Function that determines line equation from two given points

        #. Usage: slope, intercept = pttoline(ptone, pttwo)
        #. Input: ptone = tuple of x,y coordinates, pttwo = tuple of x,y coordinates
        #. Output: slope, intercept
        """
        import numpy as np

        if ptone is None: ptone = self.topleft
        if pttwo is None: pttwo = self.topright

        x0, y0 = ptone
        x1, y1 = pttwo

        linex = [x0, x1]
        liney = [y0, y1]
        self.slope, self.icept = np.polyfit(linex, liney, 1)
        self.log.ilog('Fitted line intercept: %g, slope: %g' %(self.icept, self.slope))

        return self.slope, self.icept

    def searchdots(self, boxwidth = None, topleft = None, topright = None, bottomleft = None, stepsize = None):
        """
        * Function to determine coordinates of dots by center of gravity measurements

        #. Usage: dotx, doty = (boxwidth, topleft, topright, bottomleft, stepsize)
        #. Input: boxwidth width of integration, topleft/topright/bottomleft coordinate pair
        #. Output: dotx and doty = list of X and Y coordinates of dots

        """
        from math import cos, sin, atan, sqrt
        self.log.fcttolog()

        if boxwidth is None: boxwidth = self.boxwidth
        if topleft is None: topleft = self.topleft
        if topright is None: topright = self.topright
        if bottomleft is None: bottomleft = self.bottomleft
        if stepsize is None: stepsize = self.stepsize

        self.pttoline()

        self.no_cols = int(round(sqrt((topleft[1]-topright[1])**2+(topleft[0]-topright[0])**2)/(2500/stepsize)))
        self.no_rows = int(round(sqrt((topleft[1]-bottomleft[1])**2+(topleft[0]-bottomleft[0])**2)/(2500/stepsize)))
        self.log.ilog('Number of dot columns detected: %g' %self.no_cols)
        self.log.ilog('Number of dot rows detected: %g' %self.no_rows)

        def cog_dots(img, x0, y0, slope, stepsize, boxwidth, no_cols, no_rows):
            xoffset = int(-self.nx/2 + x0 + cos(atan(slope))*eachCol*2500/stepsize + \
            sin(atan(slope))*eachRow*2500/stepsize + 0.5)
            yoffset = int(-self.ny/2 + y0 + sin(atan(slope))*eachCol*2500/stepsize - \
            cos(atan(slope))*eachRow*2500/stepsize + 0.5)
            dotimg = Util.window(img, boxwidth, boxwidth, 1, xoffset, yoffset, 0)
            # dotimg.write_image('dotstack.spi', eachCol*no_cols+eachRow)
            self.log.dlog('X-offset: %g, Y-offset: %g' %(xoffset, yoffset))
            shift = dotimg.phase_cog()
            self.log.ilog('Offset from ideal dot at column %g and row %g : %g, %g' \
            %(eachCol, eachRow, shift[0], shift[1]))
            xcoord = x0 + shift[0] + cos(atan(slope))*eachCol*2500/stepsize + \
            sin(atan(slope))*eachRow*2500/stepsize 
            ycoord = y0 + shift[1] + sin(atan(slope))*eachCol*2500/stepsize - \
            cos(atan(slope))*eachRow*2500/stepsize 
            return xcoord, ycoord

        self.dots = [cog_dots(self.img, topleft[0], topleft[1], self.slope, stepsize, boxwidth, 
                              self.no_cols, self.no_rows) for eachCol in range(self.no_cols) 
                     for eachRow in range(self.no_rows)]

        return self.dots, self.no_cols, self.no_rows

    def dotdistance(self, dots = None, stepsize = None, no_cols = None, no_rows = None):
        """
        * Function to determine distances between adjacent dots

        #. Usage: distances = dotdistance(dots, stepsize, no_cols, no_rows)
        #. Input: dots = list of X and Y coordinates of dots, scanner stepsize, number of columns,\
        number of rows
        #. Output: list of horizontal and vertical distances from adjacent dots

        """
        from math import sqrt
        self.log.fcttolog()

        if dots is None: dots = self.dots
        if stepsize is None: stepsize = self.stepsize
        if no_cols is None: no_cols = self.no_cols
        if no_rows is None: no_rows = self.no_rows

        def calc_horid(dots, no_rows, stepsize, k):
                x0, y0 = dots[k]
                x1, y1 = dots[k+no_rows]
                dist = sqrt((y1-y0)**2 + (x1-x0)**2)*stepsize
                self.log.ilog('Horizontal distance between dot %g and dot %g determined: %g micrometer' \
                %(k, k+no_rows, dist))
                return dist

        self.horidots = [calc_horid(dots, no_rows, stepsize, k) for k in range(no_rows*(no_cols-1))]

        def calc_vertd(dots, stepsize, j):
                x0, y0 = dots[j]
                x1, y1 = dots[j+1]
                dist = sqrt((y1-y0)**2 + (x1-x0)**2)*stepsize
                if dist < 4000:
                    self.log.ilog('Vertical distance between dot %g and dot %g determined: %g micrometer'\
                    %(j, j+1, dist))
                    return dist
                else:
                    pass

        self.vertdots = [calc_vertd(dots, stepsize, j) for j in range(no_cols*no_rows-1)]
        return self.horidots, self.vertdots

    def vismic(self, nx, ny, subplot):
        """
        * Function to prepare a micrograph in row and column dimension for matplotlib

        #. Usage: plot = vismic(nx, ny, subplot)
        #. Input: nx = number of rows, ny = number of columns, subplot = idendity of subplot
        #. Output: subplot to be returned
        """
        ax1 = subplot
        self.log.fcttolog()

        for t in ax1.get_xticklabels(): t.set_fontsize(8)
        for t in ax1.get_yticklabels(): t.set_fontsize(8)
        ax1.set_xlabel('Row')
        ax1.set_ylabel('Column')
        ax1.set_xlim(0, nx)
        ax1.set_ylim(0, ny)
        ax1.axis('scaled')
        ax1.minorticks_on()
        ax1.grid(True)

        return ax1

    def visdots(self, dots = None, horidots = None, vertdots = None):
        """
        * Function to visualize detected dots using matplotlib

        #. Usage: output = visfit(dots, horidots, vertdots)
        #. Input: dots = list of X and Y coordinates of dots, horidots, vertdots = list of distances
        #. Output: output plot to saved to PDF, PNG format
        """
        import matplotlib.font_manager as font_manager
        self.log.fcttolog()

        if dots is None: dots = self.dots
        if horidots is None: horidots = self.horidots
        if vertdots is None: vertdots = self.vertdots

        # unpack dots
        xarr = []; yarr = []
        for xy in dots:
            xarr.append(xy[0])
            yarr.append(xy[1])

        scandotfit_plot = DiagnosticPlot()
        self.fig = scandotfit_plot.add_header_and_footer(self.feature_set)

        ax1 = self.fig.add_subplot(121)
        ax1.plot(xarr, yarr, 'x', markeredgewidth=0.2, markersize=3)
        ax1.set_title('Detected dots on micrograph')
        ax1 = self.vismic(self.nx, self.ny, ax1)

        vertdotsarr = np.array([i for i in vertdots if i is not None])/1000
        self.vertical_mean = np.mean(vertdotsarr)
        self.vertical_stdev = np.std(vertdotsarr)
        self.log.ilog('Mean distance of vertical distances: %g, standard deviation (mm): %g' %
                      (self.vertical_mean, self.vertical_stdev))
        horidotsarr = np.array([i for i in horidots if i is not None])/1000
        self.horizontal_mean = np.mean(horidotsarr)
        self.horizontal_stdev = np.std(horidotsarr)
        self.log.ilog('Mean distance of horizontal distances: %g, standard deviation (mm): %g' %
                      (self.horizontal_mean, self.horizontal_stdev))

        ax2 = self.fig.add_subplot(122)
        ax2.set_title('2.5 mm dot distances')
        ax2.hist(vertdotsarr, 40, label='Mean vertical distance: %g\nstdev: %g (mm)' %
                 (self.vertical_mean, self.vertical_stdev))
        ax2.hist(horidotsarr, 40, label='Mean horizontal distance: %g\nstdev: %g (mm)' %
                 (self.horizontal_mean, self.horizontal_stdev))
        ax2.legend(loc='best' , prop=font_manager.FontProperties(size='x-small'))
        ax2.set_xlabel('Distance (mm)')
        ax2.set_ylabel('Number of measurements')
        for t in ax2.get_xticklabels(): t.set_fontsize(8)
        for t in ax2.get_yticklabels(): t.set_fontsize(8)
        
        self.fig.savefig(self.outfile)
        return self.outfile

    def perform_scandotfit(self):
        self.log.plog(10)
        self.searchdots()
        self.log.plog(40)
        self.dotdistance()
        self.log.plog(70)
        self.visdots()
        
        self.log.endlog(self.feature_set)
        
def main():
    # Option handling
    parset = ScanDotFitPar()
    mergeparset = OptHandler(parset)

    ######## Program
    mic = ScanDotFit(mergeparset)
    mic.perform_scandotfit()

if __name__ == '__main__':
    main()



# Author: Carsten Sachse 
# Copyright: EMBL (2010 - 2018)
# License: see license.txt for details
"""
Program to simulate helical diffraction pattern and plot helical lattice from a series of indexed layer lines or \
rise/rotation parameters
"""
from EMAN2 import EMData, EMNumPy
from PyQt4.QtCore import SIGNAL, Qt
from PyQt4.QtGui import QApplication, QWidget, QLineEdit, QPushButton, QDoubleSpinBox, QLabel, QSplitter, QFont, \
    QToolTip, QComboBox, QCheckBox
from matplotlib.ticker import MultipleLocator
from scipy import special
from spring.csinfrastr.csfeatures import Features
from spring.csinfrastr.csgui import NumbersOptionsGuiWindow, QTabWidgetCloseable
from spring.csinfrastr.cslogger import Logger
from spring.csinfrastr.csproductivity import DiagnosticPlot
from spring.csinfrastr.csreadinput import OptHandler
from spring.segment2d.segmentexam import SegmentExam
from spring.segment3d.segclassreconstruct import SegClassReconstruct
from spring.springgui.springdataexplore import SpringCommon, SpringDataExplore, SpringDataExploreDraw
from tabulate import tabulate
import matplotlib.cm as cm
import matplotlib.font_manager as font_manager
import numpy as np
import sys

class SegLayer2LatticePar(object):
    """
    Class to initiate default dictionary with input parameters including help and range values and status dictionary
    """
    def __init__(self):
        # package/program identity
        self.package = 'emspring'
        self.progname = 'seglayer2lattice'
        self.proginfo = __doc__
        self.code_files = [self.progname]

        self.seglayer2lattice_features = Features()
        self.feature_set = self.seglayer2lattice_features.setup(self)
        
        self.define_parameters_and_their_properties()
        self.define_program_states()
    

    def define_parameters_and_their_properties(self):

        self.feature_set = self.seglayer2lattice_features.set_input_power_to_analyze(self.feature_set)
        
        self.feature_set = self.seglayer2lattice_features.set_input_power_spectrum(self.feature_set, 
                                                                                   'Analyze power spectrum')
        
        self.feature_set = self.seglayer2lattice_features.set_interactive_vs_batch_mode(self.feature_set)
        self.feature_set = self.seglayer2lattice_features.set_output_plot(self.feature_set, self.progname + '_diag.pdf',
                                                                          'Batch mode')
        
        self.feature_set = self.seglayer2lattice_features.set_pixelsize(self.feature_set)
        self.feature_set = self.seglayer2lattice_features.set_high_pass_filter_option(self.feature_set)
        self.feature_set = self.seglayer2lattice_features.set_low_pass_filter_option(self.feature_set)
        self.feature_set = self.seglayer2lattice_features.set_high_and_low_pass_filter_cutoff(self.feature_set)
        self.feature_set = self.set_layer_bessel_or_symmetry_choice(self.feature_set)
        self.feature_set = self.set_layer_bessel_or_symmetry(self.feature_set)
        self.feature_set = self.seglayer2lattice_features.set_rotational_symmetry(self.feature_set)
        self.feature_set = self.seglayer2lattice_features.set_exact_helix_width(self.feature_set)
        

    def define_program_states(self):
        self.feature_set.properties['Number of lines Logfile']=50
        self.feature_set.program_states['plot_layer_lines_to_power_and_lattice']='Plots assigned layer lines to ' + \
        'helical lattice'


    def set_layer_bessel_or_symmetry(self, feature_set):
        inp8 = 'Layer line/Bessel or rise/rotation pair(s)'
        feature_set.parameters[inp8] = str('(0.03, 2); (0.06, -4); (0.09, 6)')
        feature_set.hints[inp8] = 'List of comma-separated pairs in brackets. Pairs are separated by semicolon. ' + \
        '(Layer line position [1/Angstrom], Bessel order [integer]) or (Helical rise [Angstrom], rotation [degrees])'
        feature_set.properties[inp8] = feature_set.file_properties(1, ['*'], None)
        feature_set.level[inp8]='beginner'
        
        return feature_set
    

    def set_layer_bessel_or_symmetry_choice(self, feature_set):
        inp7 = 'Layer line/Bessel order or helical rise/rotation choice'
        feature_set.parameters[inp7] = str('rise/rotation')
        feature_set.hints[inp7] = 'Choose whether \'layer/bessel\', \'rise/rotation\' or \'pitch/unit_number \' ' + \
        'pairs are given for generating the helical lattice.'
        
        feature_set.properties[inp7] = feature_set.choice_properties(2, ['layer/bessel', 'rise/rotation',
        'pitch/unit_number'], 'QComboBox')
        
        feature_set.level[inp7]='beginner'

        return feature_set
    

class SegLayer2LatticeGui(QWidget):
    def __init__(self, feature_set, parent = None):
        QWidget.__init__(self, parent)
        
        self.feature_set = feature_set
        self.properties = feature_set.properties
        
        self = SpringCommon().setup_spring_page_top(self, feature_set)
        
        self.get_parameters_from_dict()
        self.sym_plot = DiagnosticPlot()

        self.build_figure()

        self.build_controls()
        self.tabWidget = QTabWidgetCloseable()
        
        self.add_tab_with_new_symmetry()
        
        self.layout.addWidget(self.tabWidget, 1, 0, 1, 5)
        
        self.setLayout(self.layout)
        
        self.setMouseTracking(True)
 

    def enterEvent(self, event):
        QToolTip.setFont(QFont('Courier', 8))

        
    def get_parameters_from_dict(self):
        self.seglayer2lattice = SegLayer2Lattice(self.feature_set)
        
        
    def build_figure(self):
        fig1 = self.sym_plot.create_next_figure()
        ax1 = fig1.add_subplot(111)
        fig2 = self.sym_plot.create_next_figure()
        ax2 = fig2.add_subplot(111)
        fig3 = self.sym_plot.create_next_figure()
        ax3 = fig3.add_subplot(111)

        layerline_bessel_pairs = self.seglayer2lattice.get_layer_line_bessel_pairs()
        ax1, ax2 = self.seglayer2lattice.plot_layer_lines_in_power_spectrum(layerline_bessel_pairs, ax1, ax2)

        title = '{0}: ({1})'.format(self.seglayer2lattice.layer_bessel_or_sympair_choice, 
                                  self.seglayer2lattice.layer_sympair_entry.strip('(').strip(')'))
        if self.seglayer2lattice.rot_sym > 1:
                title += ' C{0}'.format(int(self.seglayer2lattice.rot_sym))

        ax1.set_title(title.title())
        ax3 = self.seglayer2lattice.plot_helical_lattice(layerline_bessel_pairs, ax3)

        self.power_canvas = SpringDataExplore(fig1)
        self.bessel_canvas = SpringDataExplore(fig2)
        
        res, orders = zip(*layerline_bessel_pairs)
        res_real = np.around(np.flipud(1 / np.array(res)), decimals=2)
        res = np.flipud(np.around(np.array(res), decimals=5))
        orders = np.flipud(orders)

        self.tool_str = tabulate(zip(res, res_real, orders),
                            headers=['Resolution (1/A)', 'Resolution (A)', 'Bessel order'], 
                            tablefmt='rst')
        
        self.bessel_canvas.main_frame.setToolTip(self.tool_str)
        self.bessel_canvas.setMouseTracking(True)

        self.lattice_canvas = SpringDataExplore(fig3)

        self.splitter = QSplitter(Qt.Horizontal)
        self.splitter_vert = QSplitter(Qt.Vertical)
            
        self.splitter_vert.addWidget(self.lattice_canvas)
        self.splitter_vert.addWidget(self.bessel_canvas)
        self.splitter.addWidget(self.power_canvas)
        self.splitter.addWidget(self.splitter_vert)

        return self.splitter
        
        
    def build_controls(self):

        choices = ['rise/rotation', 'pitch/unit_number']
        self.pushbutton = QPushButton()
        self.pushbutton.setText('OK')
        self.layout.addWidget(self.pushbutton, 2, 4, 1, 1)
        self.connect(self.pushbutton, SIGNAL('clicked()'), self.open_another_tab)
        if self.seglayer2lattice.layer_bessel_or_sympair_choice in ['layer/bessel']:
            text = NumbersOptionsGuiWindow().convert_angstrom_string('Layer line position (Angstrom^-1)/Bessel order')
            self.label = QLabel()
            self.label.setText(text)
            self.lineedit = QLineEdit()
            self.lineedit.setToolTip('(' + ', '.join(text.split('/')) + '(int)); (LL, BO); ...')
            self.lineedit.setText(self.seglayer2lattice.layer_sympair_entry)
            self.layout.addWidget(self.label, 2, 0, 1, 1)
            self.label.setAlignment(Qt.AlignRight|Qt.AlignCenter)
            
            self.layout.addWidget(self.lineedit, 2, 1, 1, 2)
        elif self.seglayer2lattice.layer_bessel_or_sympair_choice in choices:
            self.inputChoice= QComboBox()
            self.inputChoice.addItems([each_choice.title() for each_choice in choices])
            if self.seglayer2lattice.layer_bessel_or_sympair_choice in ['rise/rotation']:
                text = NumbersOptionsGuiWindow().convert_angstrom_string('Helical rise (Angstrom)/Helical rotation (degrees)')
                self.inputChoice.setCurrentIndex(0)
            elif self.seglayer2lattice.layer_bessel_or_sympair_choice in ['pitch/unit_number']:
                text = NumbersOptionsGuiWindow().convert_angstrom_string('Pitch (Angstrom)/Number of units per turn')
                self.inputChoice.setCurrentIndex(1)
            
            self.inputChoice.setToolTip(text)
            rise, rotation = \
            SegLayer2Lattice().convert_entry_to_rise_rotation_pair(self.seglayer2lattice.layer_sympair_entry)
            
            self.rise_dial = QDoubleSpinBox()
            self.rise_dial.setRange(0, 1000)
            self.rise_dial.setSingleStep(1)
            self.rise_dial.setDecimals(3)
            self.rise_dial.setToolTip(text.split('/')[0])
            self.rise_dial.setValue(rise)
            
            self.rotation_dial = QDoubleSpinBox()
            self.rotation_dial.setRange(0.001, 360)
            self.rotation_dial.setSingleStep(0.001)
            self.rotation_dial.setDecimals(3)
            self.rotation_dial.setToolTip(text.split('/')[1])
            self.rotation_dial.setValue(rotation)
            
            self.rot_sym_check = QCheckBox()
            rot_sym_text = 'Rotational symmetry'
            self.rot_sym_check.setText(rot_sym_text)
            self.rot_sym_check.setToolTip(self.feature_set.hints[rot_sym_text])
            self.rot_sym_dial = QDoubleSpinBox()
            self.rot_sym_dial.setRange(1, 1000)
            self.rot_sym_dial.setSingleStep(1)
            self.rot_sym_dial.setDecimals(0)
            self.rot_sym_dial.setToolTip('Rotational symmetry')
            self.rot_sym_dial.setEnabled(False)

            self.layout.addWidget(self.inputChoice, 2, 0, 1, 1)
            self.layout.addWidget(self.rise_dial, 2, 2, 1, 1)
            self.layout.addWidget(self.rotation_dial, 2, 3, 1, 1)
            
            self.layout.addWidget(self.rot_sym_check, 3, 0, 1, 1)
            self.layout.addWidget(self.rot_sym_dial, 3, 2, 1, 2)
            
            self.connect(self.inputChoice, SIGNAL('currentIndexChanged(int)'), self.changeInputChoice)
            self.connect(self.rot_sym_check, SIGNAL('toggled(bool)'), self.turn_rot_sym_dial_on_or_off)

            if self.seglayer2lattice.rot_sym > 1:
                self.rot_sym_check.setChecked(True)
                self.rot_sym_dial.setValue(self.seglayer2lattice.rot_sym)


    def turn_rot_sym_dial_on_or_off(self, state):
        self.rot_sym_dial.setEnabled(state)


    def changeInputChoice(self, choice):
        if choice == 0:
            self.seglayer2lattice.layer_bessel_or_sympair_choice = 'rise/rotation' 
            text = NumbersOptionsGuiWindow().convert_angstrom_string('Helical rise (Angstrom)/Helical rotation (degrees)')
        if choice == 1:
            self.seglayer2lattice.layer_bessel_or_sympair_choice = 'pitch/unit_number' 
            text = NumbersOptionsGuiWindow().convert_angstrom_string('Pitch (Angstrom)/Number of units per turn')

        self.rise_dial.setToolTip(text.split('/')[0])
        self.rotation_dial.setToolTip(text.split('/')[1])

    
    def open_another_tab(self):
        if self.seglayer2lattice.layer_bessel_or_sympair_choice in ['layer/bessel']:
            self.seglayer2lattice.layer_sympair_entry = str(self.lineedit.text())
        else:
            self.seglayer2lattice.layer_sympair_entry = '({0}, {1})'.format(self.rise_dial.value(),
            self.rotation_dial.value())
            
        if hasattr(self, 'rot_sym_dial'):
            if self.rot_sym_dial.isEnabled():
                self.seglayer2lattice.rot_sym = self.rot_sym_dial.value()
            else:
                self.seglayer2lattice.rot_sym = 1
        else:
            self.seglayer2lattice.rot_sym = 1

        self.build_figure()
        self.add_tab_with_new_symmetry()
        

    def get_title_for_current_tab(self):
        title = self.seglayer2lattice.layer_sympair_entry.strip('(').strip(')')
        if hasattr(self, 'rot_sym_dial'):
            if self.rot_sym_dial.isEnabled():
                title += ' C{0}'.format(int(self.seglayer2lattice.rot_sym))
        title += ' ({0})'.format(self.seglayer2lattice.layer_bessel_or_sympair_choice)

        return title


    def add_tab_with_new_symmetry(self):
        title = self.get_title_for_current_tab()

        self.splitter.setToolTip(title.title())
        self.tabWidget.addTab(self.splitter, title.title())
        self.tabWidget.setCurrentIndex(self.tabWidget.indexOf(self.splitter))
        self.tabWidget.setTabToolTip(self.tabWidget.currentIndex(), title.title())

            
class SegLayer2LatticePreparation(object):
    """
    * Class that holds functions for plotting assigned layer lines to helical lattice

    """
    def __init__(self, parset = None):
        self.log = Logger()
        if parset is not None:
            self.feature_set = parset
            p = self.feature_set.parameters

            self.power_img = p['Power spectrum input image']
            self.analyze_power = p['Analyze power spectrum']
            self.batch_mode = p['Batch mode']
            self.outfile = p['Diagnostic plot']
            self.pixelsize = p['Pixel size in Angstrom']
            
            self.low_pass_filter_option = p['Low-pass filter option']
            self.high_pass_filter_option = p['High-pass filter option']
            self.high_pass_filter_cutoff = p['High and low-pass filter cutoffs in 1/Angstrom'][0]
            self.low_pass_filter_cutoff = p['High and low-pass filter cutoffs in 1/Angstrom'][1]
            
            if not self.high_pass_filter_option:
                self.high_pass_filter_cutoff = 1/1000.0
            if not self.low_pass_filter_option:
                self.low_pass_filter_cutoff = 1/(2*self.pixelsize)
            
            self.helixwidth = p['Precise helix width in Angstrom']
            self.helix_radius = self.helixwidth / 2.0
            self.layer_bessel_or_sympair_choice = p['Layer line/Bessel order or helical rise/rotation choice']
            self.layer_sympair_entry = p['Layer line/Bessel or rise/rotation pair(s)']
            self.rot_sym = p['Rotational symmetry']
            

    def prepare_layerline_bessel_pairs_from_list(self, layerlines, bessel_orders):
        """
        >>> from spring.segment2d.seglayer2lattice import SegLayer2Lattice
        >>> s = SegLayer2Lattice()
        >>> positions = '0.123, 0.234, 0.345'
        >>> s.prepare_layerline_bessel_pairs_from_list(positions, '-2, 4, 5')
        [(0.123, -2), (0.234, 4), (0.345, 5)]
         >>> s.prepare_layerline_bessel_pairs_from_list(positions, '-2, 4') #doctest: +NORMALIZE_WHITESPACE
         Traceback (most recent call last):
          File '<stdin>', line 1, in <module>
          File 'spring/segment2d/seglayer2lattice.py', line 85, in 
              prepare_layerline_bessel_pairs_from_list
            raise ValueError, error_message
         ValueError: Number of layer line positions does not equal number of 
             assigned Bessel orders. Please revise.
         >>> s.prepare_layerline_bessel_pairs_from_list('0.123', '-2 4') #doctest: +NORMALIZE_WHITESPACE
         Traceback (most recent call last):
          File '<stdin>', line 1, in <module>
          File 'spring/segment2d/seglayer2lattice.py', line 77, in 
              prepare_layerline_bessel_pairs_from_list
            raise ValueError, error_message
         ValueError: No comma-separated list was detected. Please check 
             input of layer line positions.
        """
        layerline_list = bessel_order_list = None
        for separator in [',', ';']:
            if layerlines.find(separator) >= 0:
                layerline_list = layerlines.split(separator)
                layerline_list = [float(each_layer_line) for each_layer_line in layerline_list]
        
            if bessel_orders.find(separator) >= 0:
                bessel_order_list = bessel_orders.split(separator)
                bessel_order_list = [int(each_bessel_order) for each_bessel_order in bessel_order_list]
        
        if layerline_list is None:
            error_message = 'No comma-separated list was detected. Please check input of layer line positions.'
            raise ValueError(error_message)
        
        if bessel_order_list is None:
            error_message = 'No comma-separated list was detected. Please check input of Bessel orders.'
            raise ValueError(error_message)
        
        if len(layerline_list) != len(bessel_order_list):
            error_message = 'Number of layer line positions does not equal number of assigned Bessel orders. Please '+ \
            'revise.'
            raise ValueError(error_message)
            
        if len(layerline_list) == 2 and len(bessel_order_list) == 2:
            self.log.wlog('Only two layer lines entered. This will not result in an unambiguous lattice. ' + \
                          'Recommended minimum number of layer lines is 3.')
            
        layerline_bessel_pairs = list(zip(layerline_list, bessel_order_list))
        
        return layerline_bessel_pairs
            

    def convert_entry_to_rise_rotation_pair(self, entry):
        """
        >>> from spring.segment2d.seglayer2lattice import SegLayer2Lattice
        >>> s = SegLayer2Lattice()
        >>> s.convert_entry_to_rise_rotation_pair('1.408, 22.03')
        (1.408, 22.03)
        """
        stripped_off_entry = entry.strip().strip(')').strip('(')
        for separator in [',', ';']:
            if stripped_off_entry.find(separator) >= 0:
                rise_rotation_pair = stripped_off_entry.split(separator)
                rise_rotation_pair = (float(rise_rotation_pair[0]), float(rise_rotation_pair[1]))
        
        return rise_rotation_pair
    

class SegLayer2LatticePlot(SegLayer2LatticePreparation):
    def generate_lattice_line(self, pitch_bessel_pair, lines_count):
        
        pitch = 1/(pitch_bessel_pair[0])
        bessel_order = pitch_bessel_pair[1]
        
        total_lattice_line = []
        
        linex = np.array([0, 360])
        linexx = np.arange(0, 361, 1)
        if bessel_order > 0:
            liney = [0, bessel_order * pitch]
        elif bessel_order < 0:
            liney = [abs(bessel_order) * pitch, 0]
        elif bessel_order == 0:
            liney = [0, 0]
                    
        polyvar = np.polyfit(linex, liney, 1)
        first_lattice_line = np.polyval(polyvar, linexx)
        
        for each_lattice_line in range(-lines_count/2, lines_count, 1):
            lattice_line = first_lattice_line + each_lattice_line * pitch

#            sys.stderr.write('\nbesselorder:{0}\neach_bessel_order:{1}'.format(bessel_order, each_lattice_line))
#            sys.stderr.write('\nlinex:{0}\nliney:{1}'.format(linex.__str__(), liney.__str__()))
            
            total_lattice_line.append(list(zip(linexx, lattice_line)))
        
        return total_lattice_line
    
    
    def determine_number_of_lattice_lines_required_for_plot(self, layerline_bessel_pairs):
        """
        >>> from spring.segment2d.seglayer2lattice import SegLayer2Lattice
        >>> s = SegLayer2Lattice()
        >>> s.determine_number_of_lattice_lines_required_for_plot([(0.123, -2), (0.234, 4), (0.35, 5)])
        84
        """
        
        layer_line_positions, bessel_orders = list(zip(*layerline_bessel_pairs))
        abs_bessel_orders = [abs(each_bessel_order) for each_bessel_order in bessel_orders]
        lines_count = 2 * int(3 / min(layer_line_positions)) / (1 / max(layer_line_positions)) * max(abs_bessel_orders)
        
        return int(round(lines_count))


    def include_lattice_lines_in_plot(self, layerline_bessel_pairs, ax1, lines_count):
        lattice_lines = []
        colors = ['b', 'g', 'r', 'c', 'm', 'y', 'k']
        if len(layerline_bessel_pairs) > len(colors):
            colors = int(len(layerline_bessel_pairs)/len(colors) + 1)*colors
            
        maximum_pitch = 0
        layer_line_positions, bessel_orders = list(zip(*layerline_bessel_pairs))
        min_pitch_ind = np.argmax(layer_line_positions)
        
        for each_layer_line_number, each_pitch_bessel_pair in enumerate(layerline_bessel_pairs):
            lattice_line = self.generate_lattice_line(each_pitch_bessel_pair, lines_count)
            each_pitch, each_bessel = each_pitch_bessel_pair
            lattice_lines.append(lattice_line)
            self.log.ilog('Layer line/Bessel order pair: {0} plotted'.format(each_pitch_bessel_pair.__str__()))
            for each_line_number, each_line in enumerate(lattice_line):
                linexx, latttice_line = list(zip(*each_line))
                if each_line_number == 0:
                    ax1.plot(linexx, latttice_line, linewidth=.2, color=colors[each_layer_line_number], 
                    label='Pitch: {0} Angstrom, Bessel order: {1}'.format(round(1 / each_pitch, 2), 
                                                                          each_bessel))
                else:
                    ax1.plot(linexx, latttice_line, linewidth=.2, color=colors[each_layer_line_number])
                if each_layer_line_number == min_pitch_ind:
                    pitch_x = np.linspace(0, 360, 30)
                    pitch_line = [each_line_number / each_pitch for each_pitchi in range(len(pitch_x))]
                    
                    ax1.plot(pitch_x, pitch_line, linewidth=.1, linestyle='dashed',
                    color=colors[each_layer_line_number])
            
            if 1 / each_pitch_bessel_pair[0] > maximum_pitch:
                maximum_pitch = 1 / each_pitch_bessel_pair[0]
        
        return maximum_pitch


    def set_up_general_lattice_plot_properties(self, ax, maximum_pitch):
        ax.legend(loc='upper center', prop=font_manager.FontProperties(size=5))
        ax.set_xlim(0, 360)
        ax.grid(True) 
        ax.yaxis.set_major_locator(MultipleLocator(20))
        ax.yaxis.set_minor_locator(MultipleLocator(10))
        ax.xaxis.set_major_locator(MultipleLocator(45))
        ax.xaxis.set_minor_locator(MultipleLocator(22.5))
        ax.set_xlabel('Angle (degrees)', fontsize=8)
        ax.set_ylabel('Distance (Angstrom)', fontsize=8)
        ax.set_ylim(0, 2 * maximum_pitch)
        
        for t in ax.get_xticklabels(): 
            t.set_fontsize(6)
        for t in ax.get_yticklabels(): 
            t.set_fontsize(6)

        return ax

    
    def limit_to_a_maximum_of_ten_for_display(self, layerline_bessel_pairs):
        pitches, orders = zip(*layerline_bessel_pairs)
        min_indices = np.argsort(np.abs(orders)).tolist()
        
        limit_ll_bessel_pairs = [layerline_bessel_pairs[each_min_order] for each_index, each_min_order in \
        enumerate(min_indices) if each_index < 10]
                
        return limit_ll_bessel_pairs
                
        
    def filter_layer_line_bessel_pairs_to_10A_and_150A_resolution(self, layerline_bessel_pairs):
        
        filtered_pairs = [layerline_bessel_pairs[each_index] for each_index, (each_pitch, each_bessel_order) in \
        enumerate(layerline_bessel_pairs) if 10 < 1/each_pitch < 150]
        
        return filtered_pairs
    

    def plot_helical_lattice(self, layerline_bessel_pairs, ax2):
        layerline_bessel_pairs = self.filter_layer_line_bessel_pairs_to_10A_and_150A_resolution(layerline_bessel_pairs)
        layerline_bessel_pairs = self.limit_to_a_maximum_of_ten_for_display(layerline_bessel_pairs)
        lines_count = self.determine_number_of_lattice_lines_required_for_plot(layerline_bessel_pairs)
        maximum_pitch = self.include_lattice_lines_in_plot(layerline_bessel_pairs, ax2, lines_count)
        self.set_up_general_lattice_plot_properties(ax2, maximum_pitch)

        return ax2


class SegLayer2LatticeMontage(SegLayer2LatticePlot):
    def histeq(self, em_img, nbr_bins=65535):
        im = np.copy(EMNumPy.em2numpy(em_img))
        #get image histogram
        imhist,bins = np.histogram(im.flatten(),nbr_bins,normed=True)
        cdf = imhist.cumsum() #cumulative distribution function
        cdf = cdf / cdf[-1] #normalize
         
        #use linear interpolation of cdf to find new pixel values
        im2 = np.interp(im.flatten(),bins[:-1], cdf)
        em_hist_img = EMNumPy.numpy2em(im2.reshape(im.shape))
         
        return em_hist_img
     
  
class SegLayer2Lattice(SegLayer2LatticeMontage):
    def compute_layer_line_bessel_pairs(self, layer_bessel_or_sympair_choice, layer_sympair_entry, helixwidth,
    pixelsize, high_res_cutoff, low_res_cutoff, rotational_sym):
        if layer_bessel_or_sympair_choice in ['rise/rotation', 'pitch/unit_number']:
            sym_numbers = self.convert_entry_to_rise_rotation_pair(layer_sympair_entry)
            
            if layer_bessel_or_sympair_choice in ['pitch/unit_number']:
                pitch, unit_number = sym_numbers
                
                rise_rotation_pair = SegClassReconstruct().convert_pitch_unit_pair_to_rise_rotation_pairs(pitch,
                unit_number)
            else:
                rise_rotation_pair = sym_numbers
            
            layerline_bessel_pairs = \
            SegClassReconstruct().generate_layerline_bessel_pairs_from_rise_and_rotation(rise_rotation_pair,
            rotational_sym, helixwidth, pixelsize, 1 / high_res_cutoff, 1 / low_res_cutoff, out_of_plane_tilt=0)
        elif layer_bessel_or_sympair_choice in ['layer/bessel']:
            layerline_bessel_pairs = Features().convert_list_of_data_pairs_from_entry_string(layer_sympair_entry)
            
        return layerline_bessel_pairs
    

    def format_ticklabels_to_six_and_xlabel_to_eight(self, ax2):
        [t.set_fontsize(6) for t in ax2.get_xticklabels() + ax2.get_yticklabels()]
        
        ax2.set_xlabel('1/Angstrom', fontsize=8)
        
        return ax2
        

    def format_second_layer_line_plot(self, ax2, line):
        ax2.set_xlim(0, max(line))
        ax2.set_yticks([])
        ax2 = self.format_ticklabels_to_six_and_xlabel_to_eight(ax2)
        
        return ax2
    

    def get_layer_line_bessel_pairs(self):
        layerline_bessel_pairs = self.compute_layer_line_bessel_pairs(self.layer_bessel_or_sympair_choice,
        self.layer_sympair_entry, self.helixwidth, self.pixelsize, self.high_pass_filter_cutoff,
        self.low_pass_filter_cutoff, self.rot_sym)

        return layerline_bessel_pairs


    def prepare_layer_line_plot(self, layerline_bessel_pairs, ax2, linex_fine):
        reciprocal_angstrom_fine = SegmentExam().make_oneoverres(linex_fine, self.pixelsize)
        for each_layer_line_pair in layerline_bessel_pairs:
            bessel_order = abs(each_layer_line_pair[1])
            bessel_function = np.abs(special.jv(bessel_order, linex_fine))
            ax2.plot(reciprocal_angstrom_fine, bessel_function)
        
        return reciprocal_angstrom_fine, ax2


    def plot_layer_lines_in_power_spectrum(self, layerline_bessel_pairs, ax1, ax2):
        if self.analyze_power:
            power_spectrum = EMData()
            power_spectrum.read_image(self.power_img, 0)
        
            power_size = power_spectrum.get_xsize()
        else:
            power_size = 200
        
        ideal_power_img, linex_fine = \
        SegClassReconstruct().prepare_ideal_power_spectrum_from_layer_lines(layerline_bessel_pairs, self.helix_radius,
        power_size, self.pixelsize)
        
#        ideal_power_img.write_image('ideal_power.hdf')
        
        if self.analyze_power:
            montage_side_by_side = SegClassReconstruct().montage_exp_vs_sim_power_spectrum(power_spectrum,
            ideal_power_img)
        else:
            montage_side_by_side = ideal_power_img
            
        montage = np.copy(EMNumPy.em2numpy(montage_side_by_side))
        
        ax1.imshow(montage, cmap=cm.jet, interpolation='nearest')
        
        nyquist = 1 / (2 * self.pixelsize)
        
        ax1 = SpringDataExploreDraw().set_adjustable_tick_values(ax1, montage_side_by_side.get_ysize(),
        montage_side_by_side.get_xsize(), (-nyquist, nyquist), (-nyquist, nyquist))
        
        ax1 = self.format_ticklabels_to_six_and_xlabel_to_eight(ax1)
        ax1.set_ylabel('1/Angstrom', fontsize=8)
        
        reciprocal_angstrom_fine, ax2 = self.prepare_layer_line_plot(layerline_bessel_pairs, ax2, linex_fine)
        ax2 = self.format_second_layer_line_plot(ax2, reciprocal_angstrom_fine)

        return ax1, ax2
        
        
    def print_seglayer2lattice(self):
        layerline_bessel_pairs = self.get_layer_line_bessel_pairs()
        
        self.log.plog(10)

        lattice_plot = DiagnosticPlot()
        
        ax1 = lattice_plot.plt.subplot2grid((2,2), (0,0), colspan=1, rowspan=1)
        ax2 = lattice_plot.plt.subplot2grid((2,2), (1,0), colspan=1, rowspan=1)
        ax3 = lattice_plot.plt.subplot2grid((2,2), (0,1), colspan=1, rowspan=2)
        
        if self.analyze_power:
            plot_title = 'Experimental vs. theoretical power spectrum' 
        else:
            plot_title = 'Theoretical power spectrum' 
            
        ax1.set_title(plot_title, fontsize=8)
        
        self.log.plog(40)
        ax1, ax2 = self.plot_layer_lines_in_power_spectrum(layerline_bessel_pairs, ax1, ax2)
        ax3 = self.plot_helical_lattice(layerline_bessel_pairs, ax3)
        self.log.plog(90)
        
        lattice_plot.fig.savefig(self.outfile, dpi=600)
        
        return lattice_plot.fig
        
        
    def launch_seglayer2lattice_gui(self, feature_set):
        self.log.fcttolog()
        app = QApplication(sys.argv)
        gridexplor = SegLayer2LatticeGui(feature_set)
        gridexplor.show()
        app.exec_()
        

    def plot_layer_lines_to_power_and_lattice(self):
        self.log.fcttolog()
        
        if self.batch_mode:
            self.print_seglayer2lattice()
        else:
            self.launch_seglayer2lattice_gui(self.feature_set)
        
        self.log.endlog(self.feature_set)
        
def main():
    # Option handling
    parset = SegLayer2LatticePar()
    mergeparset = OptHandler(parset)

    ######## Program
    layer_lines = SegLayer2Lattice(mergeparset)
    layer_lines.plot_layer_lines_to_power_and_lattice()

if __name__ == '__main__':
    main()



# Author: Carsten Sachse 08-Jun-2011
# Copyright: EMBL (2010 - 2018)
# License: see license.txt for details
"""
Program to interactively inspect 3D reconstructions from segmentrefine3d using slice viewer
"""
from EMAN2 import EMData, Util
from fundamentals import mirror
import os
from spring.csinfrastr.csfeatures import Features, FeaturesSupport
from spring.csinfrastr.cslogger import Logger
from spring.csinfrastr.csreadinput import OptHandler
from spring.segment3d.refine.sr3d_main import SegmentRefine3d
from spring.segment3d.segclassreconstruct import SegClassReconstruct
from spring.segment3d.segrefine3dinspect_gui import SegRefine3dInspectCommonOperations, SegRefine3dInspectGui
import sys

from PyQt4.QtGui import QApplication

import numpy as np


class SegRefine3dInspectPar(object):
    """
    Class to initiate default dictionary with input parameters including help and range values and status dictionary
    """
    def __init__(self):
        # package/program identity
        self.package = 'emspring'       
        self.progname = 'segrefine3dinspect'
        self.proginfo = __doc__
        self.code_files = [self.progname]

        self.segrefine3dinspect_features = Features()
        self.feature_set = self.segrefine3dinspect_features.setup(self)
        
        self.define_parameters_and_their_properties()
        self.define_program_states()


    def define_parameters_and_their_properties(self):
        self.feature_set = self.set_reconstruction_to_be_inspected(self.feature_set)
        self.feature_set = self.segrefine3dinspect_features.set_interactive_vs_batch_mode(self.feature_set)
        self.feature_set = self.set_processed_reconstruction(self.feature_set)
        
        self.feature_set = self.set_bfactor_option(self.feature_set)
        self.feature_set = self.set_bfactor_and_resolution_cutoff(self.feature_set)
        self.feature_set = self.set_sn_weighting_option(self.feature_set)
        self.feature_set = self.set_fsc_curve(self.feature_set)

        self.feature_set = self.set_real_space_mask_option(self.feature_set)
        self.feature_set = self.set_mask_type(self.feature_set)
        
        self.feature_set = self.set_layer_line_mask_option(self.feature_set)
        self.feature_set = self.segrefine3dinspect_features.set_helical_symmetry_reconstruction(self.feature_set)
        self.feature_set = self.segrefine3dinspect_features.set_helical_symmetry(self.feature_set)
        self.feature_set = self.segrefine3dinspect_features.set_rotational_symmetry(self.feature_set)
        self.feature_set = self.segrefine3dinspect_features.set_polar_apolar_helix_choice(self.feature_set)
        self.feature_set = self.set_long_helix_option(self.feature_set)
        self.feature_set = self.set_helix_length(self.feature_set)
        self.feature_set = self.set_swap_helix(self.feature_set)
        
        
    def define_program_states(self):
        self.feature_set.program_states['apply_structural_mask_if_demanded']='Generate and apply structural mask'
        self.feature_set.program_states['apply_layerline_filter_if_demanded']='Generate and apply layer-line filter'
        self.feature_set.program_states['apply_bfactor_if_demanded']='Apply B-factor'


    def set_reconstruction_to_be_inspected(self, feature_set):
        inp2 = 'Volume reconstruction'
        feature_set.parameters[inp2]='recvol_1223apix.hdf'
        feature_set.properties[inp2]=feature_set.file_properties(1,['hdf'],'getFile')
        feature_set.hints[inp2]='Untreated 3D reconstruction to be inspected: accepted image file formats ({0}).'.\
        format(', '.join(feature_set.properties[inp2].ext))
        
        feature_set.level[inp2]='beginner'
        
        return feature_set
    
    
    def set_processed_reconstruction(self, feature_set):
        inp9 = 'Output volume name'
        feature_set.parameters[inp9]='recvol.hdf'
        feature_set.properties[inp9]=feature_set.file_properties(1,['hdf'],'saveFile')
        feature_set.hints[inp9]='Output name for processed volume: accepted image file formats ({0}).'.\
        format(', '.join(feature_set.properties[inp9].ext))
        
        feature_set.level[inp9]='intermediate'
        feature_set.relatives[inp9]='Batch mode'
        
        return feature_set
    
        
    def set_interactive_vs_batch_mode(self, feature_set):
        inp3 = 'Interactive vs. batch mode'
        feature_set.parameters[inp3]='interactive'
        feature_set.hints[inp3]='Choose between interactive vs. batch inspection mode of 3D reconstruction.'
        feature_set.properties[inp3]=feature_set.choice_properties(2, ['interactive', 'batch'], 'QComboBox')
        feature_set.level[inp3]='intermediate'
        
        return feature_set
    
    
    def set_bfactor_option(self, feature_set):
        inp12 = 'B-factor'
        feature_set.parameters[inp12] = bool(True)
        feature_set.hints[inp12] = 'Option to apply a B-factor to the structure.'
        feature_set.level[inp12]='intermediate'
        feature_set.relatives[inp12]='Batch mode'

        return feature_set
    
    
    def set_bfactor_and_resolution_cutoff(self, feature_set):
        inp11 = 'B-factor and resolution cutoff'
        feature_set.parameters[inp11] = tuple((-200.0, 5.0))
        feature_set.hints[inp11] = 'B-factor in 1/Angstrom^2 and resolution cutoff in Angstrom.'
        feature_set.properties[inp11] = feature_set.Range(-10000, 10000, 0.1)
        feature_set.relatives[inp11] = (('B-factor', 'B-factor'))
        feature_set.level[inp11]='intermediate'

        return feature_set
    
    
    def set_real_space_mask_option(self, feature_set):
        inp12 = 'Real-space mask'
        feature_set.parameters[inp12] = bool(True)
        feature_set.hints[inp12] = 'Option to apply a real-space mask of choice to the structure.'
        feature_set.level[inp12]='intermediate'
        feature_set.relatives[inp12]='Batch mode'

        return feature_set
    
    
    def set_mask_type(self, feature_set):
        inp12 = 'Mask type'
        feature_set.parameters[inp12] = str('cylinder')
        feature_set.hints[inp12] = 'Type of real-space mask to apply: a cylinder mask based on the provided inner ' + \
        'and outer width of structure or a general structural mask based on thresholding.'
        
        feature_set.properties[inp12] = feature_set.choice_properties(2, ['cylinder', 'structure'],
        'QComboBox')
        
        feature_set.level[inp12]='intermediate'
        feature_set.relatives[inp12]='Real-space mask'

        return feature_set
    
    
    def set_layer_line_mask_option(self, feature_set):
        inp12 = 'Layer-line Fourier filter'
        feature_set.parameters[inp12] = bool(False)
        feature_set.hints[inp12] = 'Option to apply a Fourier filter at the positions of the layer lines.'
        feature_set.level[inp12]='expert'
        feature_set.relatives[inp12]='Batch mode'

        return feature_set
    
    
    def set_long_helix_option(self, feature_set):
        inp12 = 'Long helix'
        feature_set.parameters[inp12] = bool(False)
        feature_set.hints[inp12] = 'Option to generate a long volume with helical symmetry imposed. ' + \
        'Simultaneous application of B-factor/resolution cutoff and long helix option is discouraged as long ' + \
        'helices undergo a series of shifts/rotations requiring interpolation. This operation results to an ' + \
        'additional decay of amplitudes. Avoid additional sharpening by generating and saving the long helix ' + \
        'to a file and launch another run of {0} '.format(feature_set.progname.upper()) + \
        'to apply the B-factor and resolution cutoff to the elongated volume.'
        feature_set.level[inp12]='expert'
        feature_set.relatives[inp12]='Batch mode'

        return feature_set
    
    
    def set_sn_weighting_option(self, feature_set):
        inp12 = 'Signal-to-noise weighting'
        feature_set.parameters[inp12] = bool(False)
        feature_set.hints[inp12] = 'Option to apply a signal-to-noise weighting based on FSC curve from Rosenthal ' + \
        'and Henderson, 2003 J Mol Biol. This measure avoids amplification of noise.'
        feature_set.level[inp12]='expert'

        return feature_set
    
    
    def set_fsc_curve(self, feature_set):
        inp1 = 'FSC curve'
        feature_set.parameters[inp1] = 'fsc_curve.dat'
        feature_set.properties[inp1] = feature_set.file_properties(1, ['dat', 'db'], 'getFile')
        feature_set.hints[inp1] = 'FSC curve corresponding to provided reconstruction: ' + \
        FeaturesSupport().add_accepted_file_formats_to_hint(feature_set, inp1)
        feature_set.relatives[inp1]='Signal-to-noise weighting'
        
        feature_set.level[inp1]='expert'
        
        return feature_set


    def set_helix_length(self, feature_set):
        inp6 = 'Helix length in Angstrom'
        feature_set.parameters[inp6]=int(1000)
        feature_set.hints[inp6]='Helix length in Angstrom.'
        feature_set.properties[inp6]=feature_set.Range(0,10000,50)
        feature_set.level[inp6]='expert'
        feature_set.relatives[inp6]='Long helix'
        
        return feature_set
    
    
    def set_swap_helix(self, feature_set):
        inp12 = 'Swap handedness'
        feature_set.parameters[inp12] = bool(False)
        feature_set.hints[inp12] = 'Apply mirror operation to helix to swap handedness'
        feature_set.level[inp12]='expert'
        feature_set.relatives[inp12]='Batch mode'

        return feature_set
    
    


class SegRefine3dInspectPreparation(object):
    def __init__(self, parset = None):
        self.log = Logger()
        if parset is not None:
            self.feature_set = parset
            p = self.feature_set.parameters
            
            self.infile = p['Volume reconstruction']
            self.outfile = p['Output volume name']
            
            self.batch_mode = p['Batch mode']
            self.bfactor_option = p['B-factor']
            self.bfactor, self.resolution_cutoff = p['B-factor and resolution cutoff']
            self.sn_weighting = p['Signal-to-noise weighting']
            self.fsc_file = p['FSC curve']
            
            self.pixelsize = p['Pixel size in Angstrom']
            self.helix_inner_width, self.helixwidth = p['Estimated helix inner and outer diameter in Angstrom']
            self.real_space_mask = p['Real-space mask']
            self.mask_type = p['Mask type']
            self.layerline_filter_option = p['Layer-line Fourier filter']
            self.rise_or_pitch_choice = p['Helical rise/rotation or pitch/number of units per turn choice']
            self.helical_symmetry = p['Helical symmetry in Angstrom or degrees']
            self.rotational_symmetry = p['Rotational symmetry']
            self.polar_helix = p['Helix polarity']
            
            self.long_helix_option = p['Long helix']
            self.helix_length = p['Helix length in Angstrom']
            self.swap_helix = p['Swap handedness']
            
            self.helical_symmetry = \
            SegRefine3dInspectCommonOperations().convert_helical_symmetry_from_pitch_unit_number_to_rise_rotation(
            self.rise_or_pitch_choice, self.helical_symmetry)


    def apply_bfactor_if_demanded(self, vol, bfactor, res_cutoff, pixelsize, fsc_line):
        if self.bfactor_option:
            self.log.fcttolog()
            
            low_res_cutoff = 9.0

            if res_cutoff < 8.0:
                bfactor_det, resid = SegRefine3dInspectCommonOperations().estimate_bfactor_from_vol(vol, pixelsize, res_cutoff,
                low_res_cutoff)
                
                msg = 'The resolution range from {0} - {1} Angstrom was used for fitting the '.format(low_res_cutoff, res_cutoff) + \
                'logarithmic decay of amplitudes from the spherical average of the structure. ' + \
                'A B-factor of {0:.0f} Angstrom^2 and {1:.2f} residuals was fitted.'.format(bfactor_det, resid)
                
                self.log.ilog(msg)

            vol = SegRefine3dInspectCommonOperations().apply_bfactor_and_resolution_cutoff(vol, bfactor, res_cutoff,
            pixelsize, fsc_line)
            
        return vol
    
    
    def assemble_volume_from_image_slices(self, image_slice):
        """
        >>> from spring.segment3d.segrefine3dinspect import SegRefine3dInspect
        >>> SegRefine3dInspect().assemble_volume_from_image_slices(np.ones((3, 3)))
        array([[[ 1.,  1.,  1.],
                [ 1.,  1.,  1.],
                [ 1.,  1.,  1.]],
        <BLANKLINE>
               [[ 1.,  1.,  1.],
                [ 1.,  1.,  1.],
                [ 1.,  1.,  1.]],
        <BLANKLINE>
               [[ 1.,  1.,  1.],
                [ 1.,  1.,  1.],
                [ 1.,  1.,  1.]]])
        """
        col_count, row_count = np.shape(image_slice)
        vol = np.zeros((col_count, row_count, row_count))
        
        for each_z_plane in list(range(row_count)):
            vol[:,:,each_z_plane]=image_slice
        
        return vol
    
    
    def apply_cylinder_mask_if_demanded(self, vol):
        if self.mask_type == 'cylinder' and vol is not None or \
        vol is not None and not self.batch_mode:
            self.log.fcttolog()
            
            cyl_xsize = vol.get_xsize()
            cyl_zsize = vol.get_zsize()
            cylinder_mask = SegClassReconstruct().make_smooth_cylinder_mask(round(self.helixwidth / self.pixelsize), 
            round(self.helix_inner_width / self.pixelsize), cyl_xsize, cyl_zsize, 0.0)
        
            masked_vol = vol * cylinder_mask
        else:
            cylinder_mask = None
            masked_vol = None
        
        return masked_vol, cylinder_mask
    
            
    def apply_structural_mask_if_demanded(self, vol):
        if self.mask_type == 'structure' and vol is not None or \
        vol is not None and not self.batch_mode:
            self.log.fcttolog()
            
            structural_mask = SegmentRefine3d().build_structural_mask_from_volume(vol, round(self.helixwidth\
            / self.pixelsize), round(self.helix_inner_width / self.pixelsize), self.pixelsize, width_falloff=0.0)
            
            masked_vol = vol * structural_mask
            masked_vol.process_inplace('normalize')
        else:
            structural_mask = None
            masked_vol = None
        
        return masked_vol, structural_mask
            
    
    def apply_layerline_filter_if_demanded(self, vol, pixelsize, helical_symmetry, helixwidth):
        if self.layerline_filter_option and vol is not None:
            self.log.fcttolog()
            
            masked_vol, layer_mask = SegmentRefine3d().generate_and_apply_layerline_filter(vol, pixelsize,
            helical_symmetry, self.rotational_symmetry, helixwidth)
            masked_vol.process_inplace('normalize')
        else:
            masked_vol = None
            layer_mask = None
            
        return masked_vol, layer_mask
    
    
class SegRefine3dInspect(SegRefine3dInspectPreparation):
    def launch_interactive_inspect_gui(self, cart_volumes):
        app = QApplication(sys.argv)
        symexplor = SegRefine3dInspectGui(self.feature_set, cart_volumes)
        symexplor.show()
        app.exec_()


    def choose_correct_volume_for_batch_mode_depending_on_options(self, original_vol, layer_vol, cylinder_vol,
    cylinder_layer_vol, structural_vol, structural_layer_vol):
        if self.layerline_filter_option and not self.real_space_mask:
            nobfactor_vol = layer_vol
        elif not self.layerline_filter_option and not self.real_space_mask:
            nobfactor_vol = original_vol
        elif not self.layerline_filter_option and self.mask_type == 'cylinder':
            nobfactor_vol = cylinder_vol
        elif self.layerline_filter_option and self.mask_type == 'cylinder':
            nobfactor_vol = cylinder_layer_vol
        elif not self.layerline_filter_option and self.mask_type == 'structure':
            nobfactor_vol = structural_vol
        elif self.layerline_filter_option and self.mask_type == 'structure':
            nobfactor_vol = structural_layer_vol
        
        return nobfactor_vol


    def check_whether_pixelsize_set_in_volume(self, original_vol):
        if original_vol.has_attr('apix_x') and original_vol.has_attr('apix_y') and original_vol.has_attr('apix_z'):
            if original_vol.get_attr('apix_x') != 1.0:
                self.pixelsize = original_vol.get_attr('apix_x')
                self.feature_set.parameters['Pixel size in Angstrom'] = self.pixelsize
        

    def check_whether_volume_is_square_otherwise_pad(self):
        original_vol = EMData()
        original_vol.read_image(self.infile)
        original_vol.process_inplace('normalize')
        
        xdim, ydim, zdim = original_vol.get_xsize(), original_vol.get_ysize(), original_vol.get_zsize()
        if xdim == ydim == zdim:
            vol = original_vol
        else:
            maxdim = max(xdim, ydim, zdim)
            mindim = min(xdim, ydim, zdim)
            vol = Util.pad(original_vol, maxdim, maxdim, maxdim)
            if self.sn_weighting:
                fsc_line = SegRefine3dInspectCommonOperations().read_fsc_line_from_file(self.fsc_file, mindim)
                
                fsc_line = SegRefine3dInspectCommonOperations().read_fsc_line_from_file(self.fsc_file, maxdim,
                pad_dim=True)
                
                xvals = np.arange(len(fsc_line))
                int_xvals = np.linspace(xvals[0], xvals[-1], maxdim / 2)
                int_fsc_line = np.interp(int_xvals, xvals, fsc_line)
                
                fsc_named_tuple = SegmentRefine3d().make_fsc_line_named_tuple()

                fsc_lines = fsc_named_tuple(int_fsc_line, int_fsc_line, len(int_fsc_line) * [None], 
                len(int_fsc_line) * [None])

                self.fsc_file = os.path.splitext(self.fsc_file)[0] + '_pad' + os.extsep + 'dat'
                SegmentRefine3d().write_out_file_of_fsc_lines(fsc_lines, self.pixelsize, self.fsc_file)
                
                self.feature_set.parameters['FSC curve']=self.fsc_file

        return vol


    def launch_inspection_of_reconstruction(self):
        original_vol = self.check_whether_volume_is_square_otherwise_pad()
        self.check_whether_pixelsize_set_in_volume(original_vol)
                
        layer_vol, layer_mask = self.apply_layerline_filter_if_demanded(original_vol, self.pixelsize,
        self.helical_symmetry, self.helixwidth)
        
        self.log.plog(10)
        structural_layer_vol, structural_mask = self.apply_structural_mask_if_demanded(layer_vol)
        self.log.plog(20)
        structural_vol, structural_mask = self.apply_structural_mask_if_demanded(original_vol)
        self.log.plog(30)
        cylinder_layer_vol, cylinder_mask = self.apply_cylinder_mask_if_demanded(layer_vol)
        self.log.plog(40)
        cylinder_vol, cylinder_mask = self.apply_cylinder_mask_if_demanded(original_vol)
        self.log.plog(50)
        
        if self.batch_mode:
            nobfactor_vol = self.choose_correct_volume_for_batch_mode_depending_on_options(original_vol, layer_vol,
            cylinder_vol, cylinder_layer_vol, structural_vol, structural_layer_vol)
            
            if self.sn_weighting:
                fsc_line = SegRefine3dInspectCommonOperations().read_fsc_line_from_file(self.fsc_file,
                original_vol.get_xsize())
            else:
                fsc_line = None

            vol = self.apply_bfactor_if_demanded(nobfactor_vol, self.bfactor,
            self.resolution_cutoff, self.pixelsize, fsc_line)
            
            vol.process_inplace('normalize')
            if self.long_helix_option:
                vol = SegRefine3dInspectCommonOperations().prepare_long_helix(vol, self.helix_length, self.helixwidth,
                self.pixelsize, self.helical_symmetry, self.rotational_symmetry, self.polar_helix)

            if self.swap_helix:
                vol = mirror(vol)
                
            vol = SegClassReconstruct().set_isotropic_pixelsize_in_volume(self.pixelsize, vol)
            vol.write_image(self.outfile)
            self.log.plog(90)
        elif not self.batch_mode:
            volume_collection = SegRefine3dInspectCommonOperations().make_volume_collection_named_tuple()
            
            cart_volumes = volume_collection(original_vol, layer_vol, cylinder_mask, structural_mask)
            
            self.log.plog(90)
            self.launch_interactive_inspect_gui(cart_volumes)
        
        self.log.endlog(self.feature_set)
        
            
def main():
    # Option handling
    parset = SegRefine3dInspectPar()
    mergeparset = OptHandler(parset)

    ######## Program
    reconstruction = SegRefine3dInspect(mergeparset)
    reconstruction.launch_inspection_of_reconstruction()

if __name__ == '__main__':
    main()


#!/usr/bin/env python
"""
Test module to check segrefine3dplot
"""
from glob import glob
from spring.csinfrastr.csproductivity import TestingDirectory
from spring.segment3d.segrefine3dplot import SegRefine3dPlot, SegRefine3dPlotPar
from spring.tests.csinfrastr.test_csreadinput import EndToEndTest
from spring.tests.segment2d.test_segmentplot import TestSegmentPlotPrepare
from spring.tests.segment3d.test_segmentrefine3d import TestSegmentRefine3dMain
import os


class TestSegRefine3dPlotPrepare(object):
    def setup_segmentrefine3d(self):
        self.test_segmentrefine3d = TestSegmentRefine3dMain()
        self.test_segmentrefine3d.setup()
        self.test_segmentrefine3d.do_test_case_sr1()
        os.rename('spring.db', os.path.join(os.pardir, 'spring.db'))
        os.rename('refinement006.db', os.path.join(os.pardir, 'refinement.db'))
        

class TestSegRefine3dPlot(TestSegRefine3dPlotPrepare, SegRefine3dPlot):
    def setup(self):
        self.testingdir = TestingDirectory(os.path.splitext(os.path.basename(__file__))[0])
        self.testingdir.create()

        self.setup_segmentrefine3d()
        
        self.feature_set = SegRefine3dPlotPar()
        self.feature_set.parameters['refinement.db file']=os.pardir + os.sep + 'refinement.db'
        self.feature_set.parameters['Diagnostic plot prefix'] = 'test_diagnostic.png'
        self.feature_set.parameters['spring.db file']=os.pardir + os.sep + 'spring.db'
        self.feature_set.parameters['Refinement quantities'] = 'coordinates_subunit'#'in-plane_rotation'#'coordinates'#
        self.feature_set.parameters['Batch mode']=True#False#
        self.feature_set.parameters['Set size']='helix'
        self.feature_set = TestSegmentPlotPrepare().setup_selection_criteria_micrograph_and_helix(self.feature_set)
        
        super(TestSegRefine3dPlot, self).__init__(self.feature_set)
        
        
    def teardown(self):
        self.test_segmentrefine3d.teardown()
        os.remove('refinement.db')
        diagnostic_files = glob('test_diagnosti*.png')
        for each_file in diagnostic_files:
            os.remove(each_file)
        
        self.testingdir.remove()
        
        
class TestSegRefine3dPlotMain(TestSegRefine3dPlot):
    def do_test_case_sr3dp1(self):
        self.plot_desired_ref_quantities()
        

class TestSegRefine3dPlotMore(TestSegRefine3dPlot):
    def do_test_case_sr3dp2(self):
        self.feature_set.parameters['Refinement quantities'] = 'in-plane_rotation'
        super(TestSegRefine3dPlot, self).__init__(self.feature_set)
        
        self.plot_desired_ref_quantities()
        
        self.do_case_sr3dp3()
        self.do_case_sr3dp3a()
        self.do_case_sr3dp4()
        self.do_case_sr3dp5()
        self.do_case_sr3dp6()
        self.do_case_sr3dp7()
        self.do_case_sr3dp8()
        self.do_case_sr3dp9()
        self.do_case_sr3dp10()
        self.do_case_sr3dp11()
        self.do_case_sr3dp12()
        self.do_case_sr3dp13()
        self.do_case_sr3dp14()

    def do_case_sr3dp3(self):
        self.feature_set.parameters['Refinement quantities'] = 'normalized_in-plane_rotation'
        super(TestSegRefine3dPlot, self).__init__(self.feature_set)
        
        self.plot_desired_ref_quantities()
        
        
    def do_case_sr3dp3a(self):
        self.feature_set.parameters['Refinement quantities'] = 'out-of-plane_tilt'
        super(TestSegRefine3dPlot, self).__init__(self.feature_set)
        
        self.plot_desired_ref_quantities()
        
        
    def do_case_sr3dp4(self):
        self.feature_set.parameters['Refinement quantities'] = 'phi'
        super(TestSegRefine3dPlot, self).__init__(self.feature_set)
        
        self.plot_desired_ref_quantities()
        
        
    def do_case_sr3dp5(self):
        self.feature_set.parameters['Refinement quantities'] = 'theta'
        super(TestSegRefine3dPlot, self).__init__(self.feature_set)
        
        self.plot_desired_ref_quantities()
        
        
    def do_case_sr3dp6(self):
        self.feature_set.parameters['Refinement quantities'] = 'psi'
        super(TestSegRefine3dPlot, self).__init__(self.feature_set)
        
        self.plot_desired_ref_quantities()
        
        
    def do_case_sr3dp7(self):
        self.feature_set.parameters['Set size'] = 'micrograph'
        super(TestSegRefine3dPlot, self).__init__(self.feature_set)
        
        self.plot_desired_ref_quantities()
        
        
    def do_case_sr3dp8(self):
        self.feature_set.parameters['Set size'] = 'data_set'
        super(TestSegRefine3dPlot, self).__init__(self.feature_set)
        
        self.plot_desired_ref_quantities()
        
        
    def do_case_sr3dp9(self):
        self.feature_set.parameters['Refinement quantities'] = 'x_shift'
        super(TestSegRefine3dPlot, self).__init__(self.feature_set)
        
        self.plot_desired_ref_quantities()
        
        
    def do_case_sr3dp10(self):
        self.feature_set.parameters['Refinement quantities'] = 'y_shift'
        super(TestSegRefine3dPlot, self).__init__(self.feature_set)
        
        self.plot_desired_ref_quantities()
        
        
    def do_case_sr3dp11(self):
        self.feature_set.parameters['Refinement quantities'] = 'shift_perpendicular_to_helix'
        super(TestSegRefine3dPlot, self).__init__(self.feature_set)
        
        self.plot_desired_ref_quantities()
        
        
    def do_case_sr3dp12(self):
        self.feature_set.parameters['Refinement quantities'] = 'shift_along_helix'
        super(TestSegRefine3dPlot, self).__init__(self.feature_set)
        
        self.plot_desired_ref_quantities()
        
        
    def do_case_sr3dp13(self):
        self.feature_set.parameters['Refinement quantities'] = 'ccc_peak'
        super(TestSegRefine3dPlot, self).__init__(self.feature_set)
        
        self.plot_desired_ref_quantities()
        
        
    def do_case_sr3dp14(self):
        self.feature_set.parameters['Refinement quantities'] = 'coordinates_subunit'
        super(TestSegRefine3dPlot, self).__init__(self.feature_set)
        
        self.plot_desired_ref_quantities()
        
        
class TestSegRefine3dPlotEndToEnd(TestSegRefine3dPlot):
    def do_end_to_end_test_sp_inputfile(self):
        EndToEndTest().do_end_to_end_inputfile(self.feature_set)
 

    def do_end_to_end_test_sp_prompt(self):
        EndToEndTest().do_end_to_end_prompt(self.feature_set)
        
    
def main():
    tsr3dp = TestSegRefine3dPlotMain()
    tsr3dp.setup()
    tsr3dp.do_test_case_sr3dp1()
        
if __name__ == '__main__':
    main()
